#include <algorithm>
#include <android/log.h>
bool running = false;
extern int saveMenu_n_savestate;

#include "menu_main.cpp"

/**
 * This is an example that shows of the widgets present in
 * Guichan. The example uses the SDL back end.
 */

//#if defined(ANDROID)
//#include <SDL_screenkeyboard.h>
//#endif

#include <guichan.hpp>
#include <iostream>
#include <sstream>
//#include "menu.h"
#include <SDL/SDL_ttf.h>
//#include<SDL.h>

// Here we store a global Gui object.  We make it global
// so it's easily accessable. Of course, global variables
// should normally be avioded when it comes to OOP, but
// this examples is not an example that shows how to make a 
// good and clean C++ application but merely an example
// that shows how to use Guichan.
namespace globals
{
    gcn::Gui* gui;
}

// Include code to set up an SDL application with Guichan.
// The sdl.hpp file is responsible for creating and deleting
// the global Gui object.

/*
 * Code that sets up an SDL application with Guichan using the 
 * Guichan SDL back end.
 */

#include <guichan/sdl.hpp>
#include <guichan/contrib/sdl/sdltruetypefont.hpp>

//bool running = true;
extern char *screenshot_filename;
#define extterms files[q].size()>=4 && files[q].substr(files[q].size()-4)
#define floppyterms extterms!=".adf" && extterms!=".ADF" && extterms!=".adz" && extterms!=".ADZ" && extterms!="f.gz" && extterms!="F.GZ"
#define hddirterms extterms!=".hdf" && extterms!=".HDF"

namespace sdl
{
    SDL_Surface* screen;

    // All back ends contain objects to make Guichan work on a
    // specific target - in this case SDL - and they are a Graphics
    // object to make Guichan able to draw itself using SDL, an
    // input objec to make Guichan able to get user input using SDL
    // and an ImageLoader object to make Guichan able to load images
    // using SDL.
    gcn::SDLGraphics* graphics;
    gcn::SDLInput* input;
    gcn::SDLImageLoader* imageLoader;

    /**
     * Initialises the SDL application. This function creates the global
     * Gui object that can be populated by various examples.
     */
    void init()
    {
        // We simply initialise SDL as we would do with any SDL application.
//        SDL_Init(SDL_INIT_VIDEO);
        screen = SDL_SetVideoMode(640, 480, 16, SDL_SWSURFACE);
        // We want unicode for the SDLInput object to function properly.
        SDL_EnableUNICODE(1);
        // We also want to enable key repeat.
        SDL_EnableKeyRepeat(SDL_DEFAULT_REPEAT_DELAY, SDL_DEFAULT_REPEAT_INTERVAL);
	// Enable Android multitouch
	SDL_InitSubSystem(SDL_INIT_JOYSTICK);
	SDL_JoystickOpen(0);	
	
        // Now it's time to initialise the Guichan SDL back end.

        imageLoader = new gcn::SDLImageLoader();
        // The ImageLoader Guichan should use needs to be passed to the Image object
        // using a static function.
        gcn::Image::setImageLoader(imageLoader);
        graphics = new gcn::SDLGraphics();
        // The Graphics object needs a target to draw to, in this case it's the
        // screen surface, but any surface will do, it doesn't have to be the screen.
        graphics->setTarget(screen);
        input = new gcn::SDLInput();

        // Now we create the Gui object to be used with this SDL application.
        globals::gui = new gcn::Gui();
        // The Gui object needs a Graphics to be able to draw itself and an Input
        // object to be able to check for user input. In this case we provide the
        // Gui object with SDL implementations of these objects hence making Guichan
        // able to utilise SDL.
        globals::gui->setGraphics(graphics);
        globals::gui->setInput(input);
    }

    /**
     * Halts the SDL application.
     */
    void halt()
    {
        delete globals::gui;

        delete imageLoader;
        delete input;
        delete graphics;

//        SDL_Quit();
    }

    /**
     * Runs the SDL application.
     */
    void run()
    {
        // The main loop
        while(running)
        {
            // Check user input
            SDL_Event event;
            while(SDL_PollEvent(&event))
            {
                if (event.type == SDL_KEYDOWN)
                {
                    if (event.key.keysym.sym == SDLK_ESCAPE)
                    {
			mainMenu_case=1;
                        running = false;
                    }
                    if (event.key.keysym.sym == SDLK_LCTRL)
                    {
		      if (emulating)
			{
			  mainMenu_case=1;
			  running = false;
			  update_display();
			}
                    }
                    if (event.key.keysym.sym == SDLK_f)
                    {
                        if (event.key.keysym.mod & KMOD_CTRL)
                        {
                            // Works with X11 only
                            SDL_WM_ToggleFullScreen(screen);
                        }
                    }
                }
                else if(event.type == SDL_QUIT)
                {
                    running = false;
                }
		/*
		 * Now that we are done polling and using SDL events we pass
		 * the leftovers to the SDLInput object to later be handled by
		 * the Gui. (This example doesn't require us to do this 'cause a
		 * label doesn't use input. But will do it anyway to show how to
		 * set up an SDL application with Guichan.)
		 */
		if (event.type == SDL_MOUSEMOTION ||
			event.type == SDL_MOUSEBUTTONDOWN ||
			event.type == SDL_MOUSEBUTTONUP)
		{
			// Filter emulated mouse events for Guichan, we wand absolute input
		}
		else
		{
			// Convert multitouch event to SDL mouse event
			static int x = 0, y = 0, buttons = 0, wx=0, wy=0, pr=0;
			SDL_Event event2;
			memcpy(&event2, &event, sizeof(event));
			if (event.type == SDL_JOYBALLMOTION &&
				event.jball.which == 0 &&
				event.jball.ball == 0)
			{
				event2.type = SDL_MOUSEMOTION;
				event2.motion.which = 0;
				event2.motion.state = buttons;
				event2.motion.xrel = event.jball.xrel - x;
				event2.motion.yrel = event.jball.yrel - y;
				if (event.jball.xrel!=0)
				{
				  x = event.jball.xrel;
				  y = event.jball.yrel;
				}
				event2.motion.x = x;
				event2.motion.y = y;
				//__android_log_print(ANDROID_LOG_INFO, "GUICHAN","Mouse motion %d %d btns %d", x, y, buttons);
				if (buttons == 0)
				{
					// Push mouse motion event first, then button down event
					input->pushInput(event2);
					buttons = SDL_BUTTON_LMASK;
					event2.type = SDL_MOUSEBUTTONDOWN;
					event2.button.which = 0;
					event2.button.button = SDL_BUTTON_LEFT;
					event2.button.state =  SDL_PRESSED;
					event2.button.x = x;
					event2.button.y = y;
					//__android_log_print(ANDROID_LOG_INFO, "GUICHAN","Mouse button %d coords %d %d", buttons, x, y);
				}
			}
			if (event.type == SDL_JOYBUTTONUP &&
				event.jbutton.which == 0 &&
				event.jbutton.button == 0)
			{
				// Do not push button down event here, because we need mouse motion event first
				buttons = 0;
				event2.type = SDL_MOUSEBUTTONUP;
				event2.button.which = 0;
				event2.button.button = SDL_BUTTON_LEFT;
				event2.button.state = SDL_RELEASED;
				event2.button.x = x;
				event2.button.y = y;
				//__android_log_print(ANDROID_LOG_INFO, "GUICHAN","Mouse button %d coords %d %d", buttons, x, y);
			}
			input->pushInput(event2);
		}
	    }
            // Now we let the Gui object perform its logic.
            globals::gui->logic();
            // Now we let the Gui object draw itself.
            globals::gui->draw();
            // Finally we update the screen.
            SDL_Flip(screen);
        }
    }
}


// Include code to set up a Guichan GUI with all the widgets
// of Guichan. The code populates the global Gui object.

/**
 * Code to populate a global Gui object with all the widgets
 * of Guichan.
 */

namespace widgets
{
//    gcn::ImageFont* font;
    gcn::contrib::SDLTrueTypeFont* font;
    gcn::Container* top;
    gcn::Label* label;
    gcn::Icon* icon;
    gcn::Button* button;
    gcn::TextField* textField;
    gcn::TextBox* textBox;
    gcn::ScrollArea* textBoxScrollArea;
    gcn::ListBox* listBox;
    gcn::ScrollArea* listBoxScrollArea;
    gcn::DropDown* dropDown;
    gcn::CheckBox* checkBox1;
    gcn::CheckBox* checkBox2;
    gcn::Button* button_presetA500;
    gcn::Button* button_presetA1200;
    gcn::RadioButton* radioButton_cpu68000;
    gcn::RadioButton* radioButton_cpu68020;
    gcn::RadioButton* radioButton_chipsetocs;
    gcn::RadioButton* radioButton_chipsetecs;
    gcn::RadioButton* radioButton_chipsetaga;
    gcn::RadioButton* radioButton_kick12;
    gcn::RadioButton* radioButton_kick13;
    gcn::RadioButton* radioButton_kick20;
    gcn::RadioButton* radioButton_kick31;
    gcn::RadioButton* radioButton_cpuspeed_500;
    gcn::RadioButton* radioButton_cpuspeed_500T;
    gcn::RadioButton* radioButton_cpuspeed_1200;
    gcn::RadioButton* radioButton_cpuspeed_1200T;
    gcn::RadioButton* radioButton_cpuspeed_1200T2; 
    gcn::Slider* slider;
    gcn::Image *image;
    gcn::Window *window;
    gcn::TabbedArea* tabbedArea;
    gcn::Button* tabOneButton;
    gcn::CheckBox* tabTwoCheckBox;
    gcn::Window *window_main;
    gcn::Window *window_floppy;
    gcn::Window *window_hard;
    gcn::Window *window_display;
    gcn::Window *window_sound;
    gcn::Window *window_preset;
    gcn::Window *window_cpu;
    gcn::Window *window_chipset;
    gcn::Window *window_kickstart;
    gcn::Window *window_cpuspeed;
    gcn::Button* button_reset;
    gcn::Button* button_resume;
    gcn::Button* button_quit;
    gcn::Button* button_save_config;
    gcn::Button* button_save_config_for_game;
    gcn::Icon* icon_logo;
    gcn::Image *image_logo;
    gcn::Window *window_warning;
    gcn::Label* label_text;
    gcn::Window *window_floppy_number;    
    gcn::RadioButton* radioButton_floppy1;
    gcn::RadioButton* radioButton_floppy2;
    gcn::RadioButton* radioButton_floppy3;
    gcn::RadioButton* radioButton_floppy4;
    gcn::Button* button_ejectallfloppy;
    gcn::Button* button_df0;
    gcn::Button* button_df1;
    gcn::Button* button_df2;
    gcn::Button* button_df3;
    gcn::TextField* textField_df0;
    gcn::TextField* textField_df1;
    gcn::TextField* textField_df2;
    gcn::TextField* textField_df3;
    gcn::Window *window_floppy_speed;
    gcn::RadioButton* radioButton_floppy_speed1x;
    gcn::RadioButton* radioButton_floppy_speed2x;
    gcn::RadioButton* radioButton_floppy_speed4x;
    gcn::RadioButton* radioButton_floppy_speed8x;
    gcn::Window *window_load;
    gcn::Button* button_ok;
    gcn::Button* button_cancel;
    gcn::Button* button_select;
    gcn::Button* button_open;
    gcn::RadioButton* radioButton_chipmem_512K;
    gcn::RadioButton* radioButton_chipmem_1M;
    gcn::RadioButton* radioButton_chipmem_2M;
    gcn::RadioButton* radioButton_chipmem_4M;
    gcn::RadioButton* radioButton_chipmem_8M;
    gcn::RadioButton* radioButton_slowmem_off;
    gcn::RadioButton* radioButton_slowmem_512K;
    gcn::RadioButton* radioButton_slowmem_1M;
    gcn::RadioButton* radioButton_slowmem_15M;
    gcn::RadioButton* radioButton_fastmem_off;
    gcn::RadioButton* radioButton_fastmem_1M;
    gcn::RadioButton* radioButton_fastmem_2M;
    gcn::RadioButton* radioButton_fastmem_4M;
    gcn::RadioButton* radioButton_fastmem_8M;
    gcn::Window *window_memory;
    gcn::Window *window_chipmem;
    gcn::Window *window_slowmem;
    gcn::Window *window_fastmem;
    gcn::Window *window_boothd;
    gcn::RadioButton* radioButton_boothd_off;
    gcn::RadioButton* radioButton_boothd_hddir;
    gcn::RadioButton* radioButton_boothd_hdfile;
    gcn::Button* button_hddir;
    gcn::Button* button_hdfile;
    gcn::TextField* textField_hddir;
    gcn::TextField* textField_hdfile;
    gcn::Button* button_save_config_for_hd;
    gcn::Icon* icon_winlogo;
    gcn::Image *image_winlogo;
    gcn::Window *window_savestates;
    gcn::Window *window_control;
    gcn::Window *window_custom_control;
    gcn::RadioButton* radioButton_visibleAreaWidth_320;
    gcn::RadioButton* radioButton_visibleAreaWidth_640;
    gcn::RadioButton* radioButton_visibleAreaWidth_352;
    gcn::RadioButton* radioButton_visibleAreaWidth_704;
    gcn::RadioButton* radioButton_visibleAreaWidth_384;
    gcn::RadioButton* radioButton_visibleAreaWidth_768;
    gcn::RadioButton* radioButton_displayedLines_200;
    gcn::RadioButton* radioButton_displayedLines_216;
    gcn::RadioButton* radioButton_displayedLines_240;
    gcn::RadioButton* radioButton_displayedLines_256;
    gcn::RadioButton* radioButton_displayedLines_262;
    gcn::RadioButton* radioButton_displayedLines_270;
    gcn::RadioButton* radioButton_frameskip_0;
    gcn::RadioButton* radioButton_frameskip_1;
    gcn::RadioButton* radioButton_frameskip_2;
    gcn::RadioButton* radioButton_frameskip_3;
    gcn::RadioButton* radioButton_frameskip_4;
    gcn::RadioButton* radioButton_frameskip_5;
    gcn::Window *window_width;
    gcn::Window *window_height;
    gcn::Window *window_frameskip;
    gcn::RadioButton* radioButton_sound_off;
    gcn::RadioButton* radioButton_sound_fast;
    gcn::RadioButton* radioButton_sound_accurate;
    gcn::RadioButton* radioButton_soundrate_8k;
    gcn::RadioButton* radioButton_soundrate_11k;
    gcn::RadioButton* radioButton_soundrate_22k;
    gcn::RadioButton* radioButton_soundrate_32k;
    gcn::RadioButton* radioButton_soundrate_44k;
    gcn::RadioButton* radioButton_soundmode_mono;
    gcn::RadioButton* radioButton_soundmode_stereo;
    gcn::Window *window_sound_enable;
    gcn::Window *window_sound_rate;
    gcn::Window *window_sound_mode;
    gcn::RadioButton* radioButton_savestate_1;
    gcn::RadioButton* radioButton_savestate_2;
    gcn::RadioButton* radioButton_savestate_3;
    gcn::RadioButton* radioButton_savestate_4;
    gcn::Button* button_load_savestate;
    gcn::Button* button_save_savestate;
    gcn::Window *window_savestates_number;
    gcn::Button* button_warning;
    gcn::Label* label_ss_error;
    gcn::Window *window_savestate_screen;
    gcn::Icon* icon_savestate;
    gcn::Image *image_savestate;
    gcn::RadioButton* radioButton_onscreen_hide;
    gcn::RadioButton* radioButton_onscreen_show;
    gcn::Window *window_onscreen_control;
    gcn::RadioButton* radioButton_custom_control_on;
    gcn::RadioButton* radioButton_custom_control_off;
    gcn::Window *window_custom_control_enable;
    gcn::RadioButton* radioButton_control_config_1;
    gcn::RadioButton* radioButton_control_config_2;
    gcn::RadioButton* radioButton_control_config_3;
    gcn::RadioButton* radioButton_control_config_4;
    gcn::Window *window_control_config;
    gcn::Label* label1_control_config;
    gcn::Label* label2_control_config;
    gcn::RadioButton* radioButton_dpad_custom;
    gcn::RadioButton* radioButton_dpad_joystick;
    gcn::RadioButton* radioButton_dpad_mouse;
    gcn::Window *window_dpad;
    gcn::RadioButton* radioButton_joy_port0;
    gcn::RadioButton* radioButton_joy_port1;
    gcn::RadioButton* radioButton_joy_both;
    gcn::Window *window_joystick;
    gcn::Label* label1_sound;
    gcn::Label* label2_sound;
    gcn::RadioButton* radioButton_statusln_on;
    gcn::RadioButton* radioButton_statusln_off;
    gcn::Window *window_statusln;
    gcn::RadioButton* radioButton_autofirerate_light;
    gcn::RadioButton* radioButton_autofirerate_medium;
    gcn::RadioButton* radioButton_autofirerate_heavy;
    gcn::Window *window_autofirerate;
    gcn::RadioButton* radioButton_mouseMultiplier_25;
    gcn::RadioButton* radioButton_mouseMultiplier_50;
    gcn::RadioButton* radioButton_mouseMultiplier_1;
    gcn::RadioButton* radioButton_mouseMultiplier_2;
    gcn::RadioButton* radioButton_mouseMultiplier_4;
    gcn::Window *window_mouseMultiplier;
    gcn::RadioButton* radioButton_tapDelay_10;
    gcn::RadioButton* radioButton_tapDelay_5;
    gcn::RadioButton* radioButton_tapDelay_2;
    gcn::Window *window_tapDelay;
    
void check_settings();
void showWarning(const char *msg);
void show_error(const char *msg);
void run_menuLoad_guichan(char *, int aLoadtype);
static void unraise_loadMenu2();
void checkfilename (char *currentfilename);
void check_savestate_screen();
void check_presetModeId();
int menu_load_type;
int saveMenu_case=-1;

enum { SAVE_MENU_CASE_EXIT, SAVE_MENU_CASE_LOAD_MEM, SAVE_MENU_CASE_SAVE_MEM, SAVE_MENU_CASE_LOAD_VMU, SAVE_MENU_CASE_SAVE_VMU, SAVE_MENU_CASE_CANCEL };

    
    /*
     * List boxes and drop downs need an instance of a list model
     * in order to display a list.
     */
    class DemoListModel : public gcn::ListModel
    {
    public:
        int getNumberOfElements()
        {
            return 5;
        }

        std::string getElementAt(int i)
        {
            switch(i)
            {
              case 0:
                  return std::string("zero");
              case 1:
                  return std::string("one");
              case 2:
                  return std::string("two");
              case 3:
                  return std::string("three");
              case 4:
                  return std::string("four");
              default: // Just to keep warnings away
                  return std::string("");
            }
        }
    };
    
    DemoListModel demoListModel;

    /*
     * List boxes and drop downs need an instance of a list model
     * in order to display a list.
     */
    class DirListModel : public gcn::ListModel
    {
        std::vector<std::string> dirs, files;

    public:

        DirListModel(const char * path)
        {
            changeDir(path);
        }
        int getNumberOfElements()
        {
	  if (menu_load_type != MENU_LOAD_HD_DIR)
            return dirs.size() + files.size();
	  else return dirs.size();
        }
        std::string getElementAt(int i)
        {
            if(i >= dirs.size() + files.size() || i < 0)
                return "---";
            if(i < dirs.size())
                return dirs[i];
	    if (menu_load_type != MENU_LOAD_HD_DIR)
            return files[i - dirs.size()];
        }
        void changeDir(const char * path)
        {
            __android_log_print(ANDROID_LOG_INFO, "GUICHAN", "DirListModel::changeDir() %s", path);
            dirs.clear();
            files.clear();
            DIR *dir;
            struct dirent *dent;
            dir = opendir(path);
            if(dir != NULL)
            {
                while((dent=readdir(dir))!=NULL)
                    if(dent->d_type == DT_DIR)
                        dirs.push_back(dent->d_name);
                    else
		      if (menu_load_type != MENU_LOAD_HD_DIR)
                        files.push_back(dent->d_name);
                if(dirs.size() > 0 && dirs[0] == ".")
                    dirs.erase(dirs.begin());
                closedir(dir);
            }
            if(dirs.size() == 0)
                dirs.push_back("..");

	    std::sort(dirs.begin(), dirs.end());
	    if (menu_load_type != MENU_LOAD_HD_DIR)
            std::sort(files.begin(), files.end());
	    
	    for (int q=0; q<files.size(); q++)
	    {
	      if (((menu_load_type == MENU_LOAD_FLOPPY) && floppyterms) || ((menu_load_type == MENU_LOAD_HDF) && hddirterms))
	      {
__android_log_print(ANDROID_LOG_INFO, "GUICHAN","filter remove= %s", files[q].begin());
		files.erase(files.begin()+q);
		q--;
	      }
	    }
        }
        bool isDir(int i)
        {
            if(i < dirs.size())
                return true;
            return false;
        }
    };

    DirListModel dirList(".");
    
class fileSelected: public gcn::SelectionListener
{
    public:
    void valueChanged(const gcn::SelectionEvent& event)
    {
        gcn::ListBox * box = (gcn::ListBox *) event.getSource();
        printf( "Selected file %s", box->getListModel()->getElementAt(box->getSelected()).c_str() );
    }
};

fileSelected selected;
    
    class ButtonActionListener : public gcn::ActionListener
    {
    public:
        // Implement the action function in ActionListener to recieve actions
        // The eventId tells us which widget called the action function.
        void action(const gcn::ActionEvent& actionEvent)
        {
	      int selected_item;
	      char filename[256]="";
            // Here we use the widget pointer to check which widget the action
            // originated from.
            if (actionEvent.getSource() == button_quit)
            {
#ifndef USE_SDLSOUND
			gp2x_stop_sound();
#endif
			saveAdfDir();
			leave_program();
			sync();
			exit(0);
            }
            if (actionEvent.getSource() == button_resume)
            {
	      if (emulating)
		{
		  mainMenu_case=1;
		  running = false;
		  update_display();
		}
            }
            // Here we use the event id in order to check what action occured.
            if (actionEvent.getSource() == button_reset)
            {
	      running = false;
			setCpuSpeed();
			gp2xMouseEmuOn=0;
			gp2xButtonRemappingOn=0;
			mainMenu_drives=nr_drives;
			if (kickstart!=oldkickstart) 
			{
				oldkickstart=kickstart;
				snprintf(romfile, 256, "%s/kickstarts/%s",launchDir,kickstarts_rom_names[kickstart]);
				bReloadKickstart=1;
				uae4all_init_rom(romfile);
				if (uae4all_init_rom(romfile)==-1)
				{
				    snprintf(romfile, 256, "%s/../../com.cloanto.amigaforever.essentials/files/rom/%s",launchDir,af_kickstarts_rom_names[kickstart]);			
				    uae4all_init_rom(romfile);
				} 				
			}
			reset_hdConf();
			if (emulating)
			{
				mainMenu_case=2;	
//				break;
			}
            } else
	    if (actionEvent.getSource() == button_save_config)
            {
		saveconfig(1);
		showWarning("General config file saved");
	    }
	    if (actionEvent.getSource() == button_presetA500)
            {
		mainMenu_chipMemory=1;
		mainMenu_slowMemory=0;
		mainMenu_fastMemory=0;
		kickstart=1;
		mainMenu_CPU_model=0;
		mainMenu_chipset=0;
		mainMenu_CPU_speed=0;
	    }
	    if (actionEvent.getSource() == button_presetA1200)
            {
		mainMenu_chipMemory=2;
		mainMenu_slowMemory=0;
		mainMenu_fastMemory=4;
		kickstart=3;
		mainMenu_CPU_model=1;
		mainMenu_chipset=2;
		mainMenu_CPU_speed=2;
	    }
	    if (actionEvent.getSource() == radioButton_cpu68000)
		mainMenu_CPU_model=0;
	    else if (actionEvent.getSource() == radioButton_cpu68020)
		mainMenu_CPU_model=1;
	    if (actionEvent.getSource() == radioButton_chipsetocs)
		mainMenu_chipset=0;
	    else if (actionEvent.getSource() == radioButton_chipsetecs)
		mainMenu_chipset=1;
	    else if (actionEvent.getSource() == radioButton_chipsetaga)
		mainMenu_chipset=2;
	    if (actionEvent.getSource() == radioButton_kick12)
		kickstart=0;
	    else if (actionEvent.getSource() == radioButton_kick13)
		kickstart=1;
	    else if (actionEvent.getSource() == radioButton_kick20)
		kickstart=2;
	    else if (actionEvent.getSource() == radioButton_kick31)
		kickstart=3;
	    if (actionEvent.getSource() == radioButton_cpuspeed_500)
		mainMenu_CPU_speed=0;
	    else if (actionEvent.getSource() == radioButton_cpuspeed_500T)
		mainMenu_CPU_speed=1;
	    else if (actionEvent.getSource() == radioButton_cpuspeed_1200)
		mainMenu_CPU_speed=2;
	    else if (actionEvent.getSource() == radioButton_cpuspeed_1200T)
		mainMenu_CPU_speed=3;
	    else if (actionEvent.getSource() == radioButton_cpuspeed_1200T2)
		mainMenu_CPU_speed=4;
	    if (actionEvent.getSource() == radioButton_floppy1)
		nr_drives=1;
	    else if (actionEvent.getSource() == radioButton_floppy2)
		nr_drives=2;
	    else if (actionEvent.getSource() == radioButton_floppy3)
		nr_drives=3;
	    else if (actionEvent.getSource() == radioButton_floppy4)
		nr_drives=4;
	    if (actionEvent.getSource() == radioButton_floppy_speed1x)
		mainMenu_floppyspeed=100;
	    else if (actionEvent.getSource() == radioButton_floppy_speed2x)
		mainMenu_floppyspeed=200;
	    else if (actionEvent.getSource() == radioButton_floppy_speed4x)
		mainMenu_floppyspeed=400;
	    else if (actionEvent.getSource() == radioButton_floppy_speed8x)
		mainMenu_floppyspeed=800;
	    if (actionEvent.getSource() == button_save_config_for_game)
            {
		if (saveconfig())
			showWarning("Config saved for this game");
	    }
	    if (actionEvent.getSource() == button_ejectallfloppy)
            {
		strcpy(uae4all_image_file0, "");
		strcpy(uae4all_image_file1, "");
		strcpy(uae4all_image_file2, "");
		strcpy(uae4all_image_file3, "");
	    }
	    
	    if (actionEvent.getSource() == button_df0)
	    {
	      	current_drive=0;
		mainMenu_case=MAIN_MENU_CASE_LOAD;
		run_menuLoad_guichan(currentDir, MENU_LOAD_FLOPPY);
		mainMenu_case=-1;
	    }
	    if (actionEvent.getSource() == button_df1)
	    {
	      	current_drive=1;
		mainMenu_case=MAIN_MENU_CASE_LOAD;
		run_menuLoad_guichan(currentDir, MENU_LOAD_FLOPPY);
		mainMenu_case=-1;
	    }
	    if (actionEvent.getSource() == button_df2)
	    {
	      	current_drive=2;
		mainMenu_case=MAIN_MENU_CASE_LOAD;
		run_menuLoad_guichan(currentDir, MENU_LOAD_FLOPPY);
		mainMenu_case=-1;
	    }
	    if (actionEvent.getSource() == button_df3)
	    {
	      	current_drive=3;
		mainMenu_case=MAIN_MENU_CASE_LOAD;
		run_menuLoad_guichan(currentDir, MENU_LOAD_FLOPPY);
		mainMenu_case=-1;
	    }
	    if (actionEvent.getSource() == button_hddir)
	    {
		run_menuLoad_guichan(currentDir, MENU_LOAD_HD_DIR);
		mainMenu_case=-1;
	    }
	    if (actionEvent.getSource() == button_hdfile)
	    {
		run_menuLoad_guichan(currentDir, MENU_LOAD_HDF);
		mainMenu_case=-1;
	    }

	    
	    if (actionEvent.getSource() == radioButton_chipmem_512K)
		mainMenu_chipMemory=0;
	    else if (actionEvent.getSource() == radioButton_chipmem_1M)
		mainMenu_chipMemory=1;
	    else if (actionEvent.getSource() == radioButton_chipmem_2M)
		mainMenu_chipMemory=2;
	    else if (actionEvent.getSource() == radioButton_chipmem_4M)
	    {
		mainMenu_chipMemory=3;
		mainMenu_fastMemory=0;
	    }
	    else if (actionEvent.getSource() == radioButton_chipmem_8M)
	    {
		mainMenu_chipMemory=4;
		mainMenu_fastMemory=0;
	    }
	    if (actionEvent.getSource() == radioButton_slowmem_off)
		mainMenu_slowMemory=0;
	    else if (actionEvent.getSource() == radioButton_slowmem_512K)
		mainMenu_slowMemory=1;
	    else if (actionEvent.getSource() == radioButton_slowmem_1M)
		mainMenu_slowMemory=2;
	    else if (actionEvent.getSource() == radioButton_slowmem_15M)
		mainMenu_slowMemory=3;
	    if (actionEvent.getSource() == radioButton_fastmem_off)
		mainMenu_fastMemory=0;
	    else if (actionEvent.getSource() == radioButton_fastmem_1M)
	    {
		mainMenu_fastMemory=1;
		if (mainMenu_chipMemory>2)
		  mainMenu_chipMemory=2;
	    }
	    else if (actionEvent.getSource() == radioButton_fastmem_2M)
	    {
		mainMenu_fastMemory=2;
		if (mainMenu_chipMemory>2)
		  mainMenu_chipMemory=2;
	    }
	    else if (actionEvent.getSource() == radioButton_fastmem_4M)
	    {
		mainMenu_fastMemory=3;
		if (mainMenu_chipMemory>2)
		  mainMenu_chipMemory=2;
	    }
	    else if (actionEvent.getSource() == radioButton_fastmem_8M)
	    {
		mainMenu_fastMemory=4;
		if (mainMenu_chipMemory>2)
		  mainMenu_chipMemory=2;
	    }
	    if (actionEvent.getSource() == radioButton_boothd_off)
		mainMenu_bootHD=0;
	    else if (actionEvent.getSource() == radioButton_boothd_hddir)
		mainMenu_bootHD=1;
	    else if (actionEvent.getSource() == radioButton_boothd_hdfile)
		mainMenu_bootHD=2;
	    if (actionEvent.getSource() == button_save_config_for_hd)
	    {
		if (saveconfig(2))
		 showWarning("Config saved for current HD");
	    }
	    if (actionEvent.getSource() == radioButton_visibleAreaWidth_320)
	    {
		visibleAreaWidth=320;
		
		mainMenu_displayHires=0;
	    }
	    else if (actionEvent.getSource() == radioButton_visibleAreaWidth_640)
	    {
		visibleAreaWidth=640;
		mainMenu_displayHires=1;
	    }
	    else if (actionEvent.getSource() == radioButton_visibleAreaWidth_352)
	    {
		visibleAreaWidth=352;
		mainMenu_displayHires=0;
	    }
	    else if (actionEvent.getSource() == radioButton_visibleAreaWidth_704)
	    {
		visibleAreaWidth=704;
		mainMenu_displayHires=1;
	    }
	    else if (actionEvent.getSource() == radioButton_visibleAreaWidth_384)
	    {
		visibleAreaWidth=384;
		mainMenu_displayHires=0;
	    }
	    else if (actionEvent.getSource() == radioButton_visibleAreaWidth_768)
	    {
		visibleAreaWidth=768;
		mainMenu_displayHires=1;
	    }

	    if (actionEvent.getSource() == radioButton_displayedLines_200)
		mainMenu_displayedLines=200;
	    else if (actionEvent.getSource() == radioButton_displayedLines_216)
		mainMenu_displayedLines=216;
	    else if (actionEvent.getSource() == radioButton_displayedLines_240)
		mainMenu_displayedLines=240;
	    else if (actionEvent.getSource() == radioButton_displayedLines_256)
		mainMenu_displayedLines=256;
	    else if (actionEvent.getSource() == radioButton_displayedLines_262)
		mainMenu_displayedLines=262;
	    else if (actionEvent.getSource() == radioButton_displayedLines_270)
		mainMenu_displayedLines=270;
	    
	    if (actionEvent.getSource() == radioButton_frameskip_0)
		mainMenu_frameskip=0;
	    else if (actionEvent.getSource() == radioButton_frameskip_1)
		mainMenu_frameskip=1;
	    else if (actionEvent.getSource() == radioButton_frameskip_2)
		mainMenu_frameskip=2;
	    else if (actionEvent.getSource() == radioButton_frameskip_3)
		mainMenu_frameskip=3;
	    else if (actionEvent.getSource() == radioButton_frameskip_4)
		mainMenu_frameskip=4;
	    else if (actionEvent.getSource() == radioButton_frameskip_5)
		mainMenu_frameskip=5;

	    if (actionEvent.getSource() == radioButton_sound_off)
		mainMenu_sound=0;
	    else if (actionEvent.getSource() == radioButton_sound_fast)
		mainMenu_sound=1;
	    else if (actionEvent.getSource() == radioButton_sound_accurate)
		mainMenu_sound=2;
	    if (actionEvent.getSource() == radioButton_soundrate_8k)
		sound_rate=8000;
	    else if (actionEvent.getSource() == radioButton_soundrate_11k)
		sound_rate=11025;
	    else if (actionEvent.getSource() == radioButton_soundrate_22k)
		sound_rate=22050;
	    else if (actionEvent.getSource() == radioButton_soundrate_32k)
		sound_rate=32000;
	    else if (actionEvent.getSource() == radioButton_soundrate_44k)
		sound_rate=44100;
	    if (actionEvent.getSource() == radioButton_soundmode_mono)
		mainMenu_soundStereo=0;
	    else if (actionEvent.getSource() == radioButton_soundmode_stereo)
		mainMenu_soundStereo=1;
	    if (actionEvent.getSource() == radioButton_savestate_1)
		saveMenu_n_savestate=0;
	    else if (actionEvent.getSource() == radioButton_savestate_2)
		saveMenu_n_savestate=1;
	    else if (actionEvent.getSource() == radioButton_savestate_3)
		saveMenu_n_savestate=2;
	    else if (actionEvent.getSource() == radioButton_savestate_4)
		saveMenu_n_savestate=3;
	    if (actionEvent.getSource() == button_load_savestate)
	    {
		mainMenu_case=MAIN_MENU_CASE_SAVESTATES;
		saveMenu_case=SAVE_MENU_CASE_LOAD_MEM;
	      	strcpy(savestate_filename,uae4all_image_file0);
			switch(saveMenu_n_savestate)
			{
				case 1:
					strcat(savestate_filename,"-1.asf"); break;
				case 2:
					strcat(savestate_filename,"-2.asf"); break;
				case 3:
					strcat(savestate_filename,"-3.asf"); break;
				default: 
					strcat(savestate_filename,".asf");
			}
			FILE *f=fopen(savestate_filename,"rb");
			if (f)
			{
				fclose(f);
				savestate_state = STATE_DORESTORE;
				setCpuSpeed();
				mainMenu_case=1;
				running = false;
				update_display();
			}
			else
			{
				show_error("File doesn't exist.");
			}
	    }
	    if (actionEvent.getSource() == button_save_savestate)
	    {
		mainMenu_case=MAIN_MENU_CASE_SAVESTATES;
		saveMenu_case=SAVE_MENU_CASE_SAVE_MEM;
	      	savestate_state = STATE_DOSAVE;
		setCpuSpeed();
		mainMenu_case=1;
		running = false;
		update_display();
	    }
	    
	    if (actionEvent.getSource() == radioButton_onscreen_hide)
		mainMenu_onScreen=0;
	    else if (actionEvent.getSource() == radioButton_onscreen_show)
		mainMenu_onScreen=1;
	    
	    if (actionEvent.getSource() == radioButton_custom_control_on)
		mainMenu_customControls=1;
	    else if (actionEvent.getSource() == radioButton_custom_control_off)
		mainMenu_customControls=0;

	    if (actionEvent.getSource() == radioButton_control_config_1)
		mainMenu_joyConf=0;
	    else if (actionEvent.getSource() == radioButton_control_config_2)
		mainMenu_joyConf=1;
	    else if (actionEvent.getSource() == radioButton_control_config_3)
		mainMenu_joyConf=2;
	    else if (actionEvent.getSource() == radioButton_control_config_4)
		mainMenu_joyConf=3;
	    
	    if (actionEvent.getSource() == radioButton_dpad_custom)
		mainMenu_custom_dpad=0;
	    else if (actionEvent.getSource() == radioButton_dpad_joystick)
		mainMenu_custom_dpad=1;
	    else if (actionEvent.getSource() == radioButton_dpad_mouse)
		mainMenu_custom_dpad=2;
	    if (actionEvent.getSource() == radioButton_joy_port0)
		mainMenu_joyPort=1;
	    else  if (actionEvent.getSource() == radioButton_joy_port1)
		mainMenu_joyPort=2;
	    else  if (actionEvent.getSource() == radioButton_joy_both)
		mainMenu_joyPort=0;
	    if (actionEvent.getSource() == radioButton_statusln_on)
		mainMenu_showStatus=true;
	    else if (actionEvent.getSource() == radioButton_statusln_off)
		mainMenu_showStatus=false;
	     if (actionEvent.getSource() == radioButton_autofirerate_light)
		mainMenu_autofireRate=12;
	     else if (actionEvent.getSource() == radioButton_autofirerate_medium)
		mainMenu_autofireRate=8;
	     else if (actionEvent.getSource() == radioButton_autofirerate_heavy)
		mainMenu_autofireRate=4;
	     if (actionEvent.getSource() == radioButton_mouseMultiplier_25)
		mainMenu_mouseMultiplier=25;
	     else if (actionEvent.getSource() == radioButton_mouseMultiplier_50)
		mainMenu_mouseMultiplier=50;
	     else if (actionEvent.getSource() == radioButton_mouseMultiplier_1)
		mainMenu_mouseMultiplier=1;
	     else if (actionEvent.getSource() == radioButton_mouseMultiplier_2)
		mainMenu_mouseMultiplier=2;
	     else if (actionEvent.getSource() == radioButton_mouseMultiplier_4)
		mainMenu_mouseMultiplier=4;
	     if (actionEvent.getSource() == radioButton_tapDelay_10)
		mainMenu_tapDelay=10;
	     else if (actionEvent.getSource() == radioButton_tapDelay_5)
		mainMenu_tapDelay=5;
	     else if (actionEvent.getSource() == radioButton_tapDelay_2)
		mainMenu_tapDelay=2;
	    
	    if (actionEvent.getSource() == button_warning)
		top->remove(window_warning);
	    
	    if ((actionEvent.getSource() == button_ok) || (actionEvent.getSource() == button_open))
	    {
	      selected_item=listBox->getSelected();
	      strcpy(filename, "");
	      strcat(filename, currentDir);
	      strcat(filename, "/");
	      strcat(filename, dirList.getElementAt(selected_item).c_str());
	      checkfilename(filename);
	    }
	    
	    if (actionEvent.getSource() == button_cancel)
	      unraise_loadMenu2();
	    
	    if ((actionEvent.getSource() == listBox) && (menu_load_type != MENU_LOAD_HD_DIR))
	    {
	      selected_item=listBox->getSelected();
	      strcpy(filename, "");
	      strcat(filename, currentDir);
	      strcat(filename, "/");
	      strcat(filename, dirList.getElementAt(selected_item).c_str());
	      checkfilename(filename);
	    }
	    
	    if (actionEvent.getSource() == button_select)
	    {
	      selected_item=listBox->getSelected();
	      strcpy(filename, "");
	      strcat(filename, currentDir);
	      strcat(filename, "/");
	      strcat(filename, dirList.getElementAt(selected_item).c_str());
	      if (menu_load_type == MENU_LOAD_HD_DIR)
	      {
		strcpy(uae4all_hard_dir, filename);
		make_hard_dir_cfg_line(uae4all_hard_dir);
		reset_hdConf();
		mainMenu_bootHD = 1;
		loadconfig(2);
	      }
	      unraise_loadMenu2();
	    }
	
	check_presetModeId();
	check_settings();
        }
    };

    ButtonActionListener* buttonActionListener; // A pointer to the above class
    
    /**
     * Initialises the widgets example by populating the global Gui
     * object.
     */
    void init()
    {
        // We first create a container to be used as the top widget.
        // The top widget in Guichan can be any kind of widget, but
        // in order to make the Gui contain more than one widget we
        // make the top widget a container.
        top = new gcn::Container();
        // We set the dimension of the top container to match the screen.
        top->setDimension(gcn::Rectangle(0, 0, 640, 480));
        // Finally we pass the top widget to the Gui object.
        globals::gui->setTop(top);

        // Now we load the font used in this example.
//        font = new gcn::ImageFont("data/fixedfont.bmp", " abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789.-()[]+'");

	TTF_Init();
	font = new gcn::contrib::SDLTrueTypeFont("data/FreeSans.ttf", 14);	
	// Widgets may have a global font so we don't need to pass the
        // font object to every created widget. The global font is static.
        gcn::Widget::setGlobalFont(font);

        // Now we create the widgets
/*
        label = new gcn::Label("Label");

        image = gcn::Image::load("gui-chan.bmp");
        icon = new gcn::Icon(image);

        button = new gcn::Button("Button");

        textField = new gcn::TextField("Text field");

        textBox = new gcn::TextBox("Multiline\nText box");
        textBoxScrollArea = new gcn::ScrollArea(textBox);
        textBoxScrollArea->setWidth(200);
        textBoxScrollArea->setHeight(100);
        textBoxScrollArea->setFrameSize(1);

        listBox = new gcn::ListBox(&demoListModel);
        listBox->setFrameSize(1);
        dropDown = new gcn::DropDown(&demoListModel);
        
        checkBox1 = new gcn::CheckBox("Checkbox 1");
        checkBox2 = new gcn::CheckBox("Checkbox 2");

        slider = new gcn::Slider(0, 10);
        slider->setSize(100, 10);

        window = new gcn::Window("I am a window  Drag me");
        window->setBaseColor(gcn::Color(255, 150, 200, 190));

*/

        image = gcn::Image::load("data/supersonic640.jpg");
        icon = new gcn::Icon(image);
        image_winlogo = gcn::Image::load("data/amiga-wallpaper-3.jpg");
        icon_winlogo = new gcn::Icon(image_winlogo);
        tabbedArea = new gcn::TabbedArea();
        tabbedArea->setSize(600, 300);

	button_ok = new gcn::Button("Ok");
	button_ok->setPosition(10,10);
	button_cancel = new gcn::Button("Cancel");
	button_cancel->setPosition(220,10);
	button_select = new gcn::Button("Select");
	button_select->setPosition(10,10);
	button_open = new gcn::Button("Open");
	button_open->setPosition(115,10);
	window_load = new gcn::Window("Load");
	window_load->setSize(290,270);
	window_load->add(button_cancel);
	button_warning = new gcn::Button(" Ok ");
	button_warning->setPosition(70,30);
	label_text = new gcn::Label("                  Warning                  ");
	label_ss_error = new gcn::Label("Emulation hasn't started yet.");
	label_ss_error->setPosition(10,10);
	window_warning = new gcn::Window("Warning");
	window_warning->add(button_warning);
	window_warning->setMovable(false);
	window_warning->setSize(200,80);
	listBox = new gcn::ListBox(&dirList);
	listBox->setSize(550,140);
	listBoxScrollArea = new gcn::ScrollArea(listBox);
	listBoxScrollArea->setFrameSize(1);
        listBoxScrollArea->setPosition(10,50);
	listBoxScrollArea->setSize(270,190);
	listBoxScrollArea->setScrollbarWidth(20);
	window_load->add(listBoxScrollArea);
	radioButton_cpu68000 = new gcn::RadioButton("68000 ", "radiocpugroup");
	radioButton_cpu68000->setPosition(5,10);
        radioButton_cpu68020 = new gcn::RadioButton("68020 ", "radiocpugroup");
	radioButton_cpu68020->setPosition(5,40);
	radioButton_chipsetocs = new gcn::RadioButton("OCS ", "radiochipsetgroup");
	radioButton_chipsetocs->setPosition(5,10);
	radioButton_chipsetecs = new gcn::RadioButton("ECS ", "radiochipsetgroup");	      
	radioButton_chipsetecs->setPosition(5,40);
	radioButton_chipsetaga = new gcn::RadioButton("AGA ", "radiochipsetgroup");
	radioButton_chipsetaga->setPosition(5,70);
	radioButton_kick12 = new gcn::RadioButton("1.2 ", "radiokickgroup");
	radioButton_kick12->setPosition(5,10);
	radioButton_kick13 = new gcn::RadioButton("1.3 ", "radiokickgroup");
	radioButton_kick13->setPosition(5,40);
	radioButton_kick20 = new gcn::RadioButton("2.0 ", "radiokickgroup");
	radioButton_kick20->setPosition(5,70);
	radioButton_kick31 = new gcn::RadioButton("3.1 ", "radiokickgroup");
	radioButton_kick31->setPosition(5,100);
	radioButton_cpuspeed_500 = new gcn::RadioButton("500 ", "radiocpuspeedgroup");
	radioButton_cpuspeed_500->setPosition(5,10);
	radioButton_cpuspeed_500T = new gcn::RadioButton("500T ", "radiocpuspeedgroup");
	radioButton_cpuspeed_500T->setPosition(5,40);
	radioButton_cpuspeed_1200 = new gcn::RadioButton("1200 ", "radiocpuspeedgroup");
	radioButton_cpuspeed_1200->setPosition(5,70);
	radioButton_cpuspeed_1200T = new gcn::RadioButton("1200T ", "radiocpuspeedgroup");
	radioButton_cpuspeed_1200T->setPosition(5,100);
	radioButton_cpuspeed_1200T2 = new gcn::RadioButton("1200T2 ", "radiocpuspeedgroup");
	radioButton_cpuspeed_1200T2->setPosition(5,130);
	radioButton_floppy1 = new gcn::RadioButton("1 ", "radiofloppynrgroup");
	radioButton_floppy1->setPosition(5,10);
	radioButton_floppy2 = new gcn::RadioButton("2 ", "radiofloppynrgroup");
	radioButton_floppy2->setPosition(5,40);
	radioButton_floppy3 = new gcn::RadioButton("3 ", "radiofloppynrgroup");
	radioButton_floppy3->setPosition(5,70);
	radioButton_floppy4 = new gcn::RadioButton("4 ", "radiofloppynrgroup");
	radioButton_floppy4->setPosition(5,100);
	button_df0 = new gcn::Button("DF0");
	button_df0->setPosition(10,20);
	button_df1 = new gcn::Button("DF1");
	button_df1->setPosition(10,60);
	button_df2 = new gcn::Button("DF2");
	button_df2->setPosition(10,100);
	button_df3 = new gcn::Button("DF3");
	button_df3->setPosition(10,140);
	textField_df0 = new gcn::TextField("insert disk image                                                            ");
        textField_df0->setHeight(20);
	textField_df0->setPosition(50,20);
	textField_df1 = new gcn::TextField("insert disk image                                                            ");
	textField_df1->setPosition(50,60);
	textField_df2 = new gcn::TextField("insert disk image                                                            ");
	textField_df2->setPosition(50,100);
	textField_df3 = new gcn::TextField("insert disk image                                                            ");
	textField_df3->setPosition(50,140);
	button_save_config_for_game = new gcn::Button("Save config for current game");
        button_save_config_for_game->setHeight(30);
	button_save_config_for_game->setPosition(200,200);
	button_ejectallfloppy = new gcn::Button("Eject All Drives");
        button_ejectallfloppy->setHeight(30);
	button_ejectallfloppy->setPosition(10,200);
	radioButton_floppy_speed1x = new gcn::RadioButton("1x ", "radiofloppyspeedgroup");
	radioButton_floppy_speed1x->setPosition(5,10);
	radioButton_floppy_speed2x = new gcn::RadioButton("2x ", "radiofloppyspeedgroup");
	radioButton_floppy_speed2x->setPosition(5,40);
	radioButton_floppy_speed4x = new gcn::RadioButton("4x ", "radiofloppyspeedgroup");
	radioButton_floppy_speed4x->setPosition(5,70);
	radioButton_floppy_speed8x = new gcn::RadioButton("8x ", "radiofloppyspeedgroup");
	radioButton_floppy_speed8x->setPosition(5,100);
	radioButton_chipmem_512K = new gcn::RadioButton("512Kb ", "radiochipmemgroup");
	radioButton_chipmem_512K->setPosition(5,10);
	radioButton_chipmem_1M = new gcn::RadioButton("1Mb ", "radiochipmemgroup");
	radioButton_chipmem_1M->setPosition(5,40);
	radioButton_chipmem_2M = new gcn::RadioButton("2Mb ", "radiochipmemgroup");
	radioButton_chipmem_2M->setPosition(5,70);
	radioButton_chipmem_4M = new gcn::RadioButton("4Mb ", "radiochipmemgroup");
	radioButton_chipmem_4M->setPosition(5,100);
	radioButton_chipmem_8M = new gcn::RadioButton("8Mb ", "radiochipmemgroup");
	radioButton_chipmem_8M->setPosition(5,130);
	radioButton_slowmem_off = new gcn::RadioButton("off ", "radioslowmemgroup");
	radioButton_slowmem_off->setPosition(5,10);
	radioButton_slowmem_512K = new gcn::RadioButton("512Kb ", "radioslowmemgroup");
	radioButton_slowmem_512K->setPosition(5,40);
	radioButton_slowmem_1M = new gcn::RadioButton("1Mb ", "radioslowmemgroup");
	radioButton_slowmem_1M->setPosition(5,70);
	radioButton_slowmem_15M = new gcn::RadioButton("1.5Mb ", "radioslowmemgroup");
	radioButton_slowmem_15M->setPosition(5,100);
	radioButton_fastmem_off = new gcn::RadioButton("off ", "radiofastmemgroup");
	radioButton_fastmem_off->setPosition(5,10);
	radioButton_fastmem_1M = new gcn::RadioButton("1Mb ", "radiofastmemgroup");
	radioButton_fastmem_1M->setPosition(5,40);
	radioButton_fastmem_2M = new gcn::RadioButton("2Mb ", "radiofastmemgroup");
	radioButton_fastmem_2M->setPosition(5,70);
	radioButton_fastmem_4M = new gcn::RadioButton("4Mb ", "radiofastmemgroup");
	radioButton_fastmem_4M->setPosition(5,100);
	radioButton_fastmem_8M = new gcn::RadioButton("8Mb ", "radiofastmemgroup");
	radioButton_fastmem_8M->setPosition(5,130);
	radioButton_boothd_off = new gcn::RadioButton("Off ", "radioboothdgroup");
	radioButton_boothd_off->setPosition(5,10);
	radioButton_boothd_hddir = new gcn::RadioButton("Dir ", "radioboothdgroup");
	radioButton_boothd_hddir->setPosition(5,40);
	radioButton_boothd_hdfile = new gcn::RadioButton("File ", "radioboothdgroup");
	radioButton_boothd_hdfile->setPosition(5,70);
	button_save_config_for_hd = new gcn::Button("Save config for current HD");
        button_save_config_for_hd->setHeight(30);
	button_save_config_for_hd->setPosition(200,200);
	button_hddir = new gcn::Button(" HD Dir");
	button_hddir->setPosition(100,40);
	button_hdfile = new gcn::Button("HD File");
	button_hdfile->setPosition(100,80);
	textField_hddir = new gcn::TextField("not selected                    ");
	textField_hddir->setPosition(170,40);
	textField_hdfile = new gcn::TextField("not selected                    ");
	textField_hdfile->setPosition(170,80);
	radioButton_visibleAreaWidth_320 = new gcn::RadioButton("320 ", "radiowidthgroup");
	radioButton_visibleAreaWidth_320->setPosition(5,10);
	radioButton_visibleAreaWidth_640 = new gcn::RadioButton("640 ", "radiowidthgroup");
	radioButton_visibleAreaWidth_640->setPosition(5,40);
	radioButton_visibleAreaWidth_352 = new gcn::RadioButton("352 ", "radiowidthgroup");
	radioButton_visibleAreaWidth_352->setPosition(5,70);
	radioButton_visibleAreaWidth_704 = new gcn::RadioButton("704 ", "radiowidthgroup");
	radioButton_visibleAreaWidth_704->setPosition(5,100);
	radioButton_visibleAreaWidth_384 = new gcn::RadioButton("384 ", "radiowidthgroup");
	radioButton_visibleAreaWidth_384->setPosition(5,130);
	radioButton_visibleAreaWidth_768 = new gcn::RadioButton("768 ", "radiowidthgroup");
	radioButton_visibleAreaWidth_768->setPosition(5,160);
	radioButton_displayedLines_200 = new gcn::RadioButton("200 ", "radioheightgroup");
	radioButton_displayedLines_200->setPosition(5,10);
	radioButton_displayedLines_216 = new gcn::RadioButton("216 ", "radioheightgroup");
	radioButton_displayedLines_216->setPosition(5,40);
	radioButton_displayedLines_240 = new gcn::RadioButton("240 ", "radioheightgroup");
	radioButton_displayedLines_240->setPosition(5,70);
	radioButton_displayedLines_256 = new gcn::RadioButton("256 ", "radioheightgroup");
	radioButton_displayedLines_256->setPosition(5,100);
	radioButton_displayedLines_262 = new gcn::RadioButton("262 ", "radioheightgroup");
	radioButton_displayedLines_262->setPosition(5,130);
	radioButton_displayedLines_270 = new gcn::RadioButton("270 ", "radioheightgroup");
	radioButton_displayedLines_270->setPosition(5,160);
	radioButton_frameskip_0 = new gcn::RadioButton("0 ", "radioframeskipgroup");
	radioButton_frameskip_0->setPosition(5,10);
	radioButton_frameskip_1 = new gcn::RadioButton("1 ", "radioframeskipgroup");
	radioButton_frameskip_1->setPosition(5,40);
	radioButton_frameskip_2 = new gcn::RadioButton("2 ", "radioframeskipgroup");
	radioButton_frameskip_2->setPosition(5,70);
	radioButton_frameskip_3 = new gcn::RadioButton("3 ", "radioframeskipgroup");
	radioButton_frameskip_3->setPosition(5,100);
	radioButton_frameskip_4 = new gcn::RadioButton("4 ", "radioframeskipgroup");
	radioButton_frameskip_4->setPosition(5,130);
	radioButton_frameskip_5 = new gcn::RadioButton("5 ", "radioframeskipgroup");
	radioButton_frameskip_5->setPosition(5,160);
	radioButton_sound_off = new gcn::RadioButton("off ", "radiosoundpresentgroup");
	radioButton_sound_off->setPosition(5,10);
	radioButton_sound_fast = new gcn::RadioButton("fast ", "radiosoundpresentgroup");
	radioButton_sound_fast->setPosition(5,40);
	radioButton_sound_accurate = new gcn::RadioButton("accurate ", "radiosoundpresentgroup");
	radioButton_sound_accurate->setPosition(5,70);
	radioButton_soundrate_8k = new gcn::RadioButton("8K ", "radiosoundrategroup");
	radioButton_soundrate_8k->setPosition(5,10);
	radioButton_soundrate_11k = new gcn::RadioButton("11K ", "radiosoundrategroup");
	radioButton_soundrate_11k->setPosition(5,40);
	radioButton_soundrate_22k = new gcn::RadioButton("22K ", "radiosoundrategroup");
	radioButton_soundrate_22k->setPosition(5,70);
	radioButton_soundrate_32k = new gcn::RadioButton("32K ", "radiosoundrategroup");
	radioButton_soundrate_32k->setPosition(5,100);
	radioButton_soundrate_44k = new gcn::RadioButton("44K ", "radiosoundrategroup");
	radioButton_soundrate_44k->setPosition(5,130);
	radioButton_soundmode_mono = new gcn::RadioButton("mono ", "radiosoundmodegroup");
	radioButton_soundmode_mono->setPosition(5,10);
	radioButton_soundmode_stereo = new gcn::RadioButton("stereo ", "radiosoundmodegroup");
	radioButton_soundmode_stereo->setPosition(5,40);
	radioButton_savestate_1 = new gcn::RadioButton("0 ", "radiosavestategroup");
	radioButton_savestate_1->setPosition(5,10);
	radioButton_savestate_2 = new gcn::RadioButton("1 ", "radiosavestategroup");
	radioButton_savestate_2->setPosition(5,40);
	radioButton_savestate_3 = new gcn::RadioButton("2 ", "radiosavestategroup");
	radioButton_savestate_3->setPosition(5,70);
	radioButton_savestate_4 = new gcn::RadioButton("3 ", "radiosavestategroup");
	radioButton_savestate_4->setPosition(5,100);
	button_load_savestate = new gcn::Button("Load State");
	button_load_savestate->setPosition(5,180);
	button_save_savestate = new gcn::Button("Save State");	
	button_save_savestate->setPosition(5,220);
	radioButton_onscreen_hide = new gcn::RadioButton("Hide ", "radioonscreengroup");
	radioButton_onscreen_hide->setPosition(5,10);
	radioButton_onscreen_show = new gcn::RadioButton("Show ", "radioonscreengroup");
	radioButton_onscreen_show->setPosition(70,10);
	radioButton_custom_control_off = new gcn::RadioButton("off ", "radiocustomcontrolgroup");
	radioButton_custom_control_off->setPosition(5,10);
	radioButton_custom_control_on = new gcn::RadioButton("on ", "radiocustomcontrolgroup");
	radioButton_custom_control_on->setPosition(60,10);
	radioButton_control_config_1 = new gcn::RadioButton("1 ", "radiocontrolconfiggroup");
	radioButton_control_config_1->setPosition(5,10);
	radioButton_control_config_2 = new gcn::RadioButton("2 ", "radiocontrolconfiggroup");
	radioButton_control_config_2->setPosition(50,10);
	radioButton_control_config_3 = new gcn::RadioButton("3 ", "radiocontrolconfiggroup");
	radioButton_control_config_3->setPosition(5,40);
	radioButton_control_config_4 = new gcn::RadioButton("4 ", "radiocontrolconfiggroup");
	radioButton_control_config_4->setPosition(50,40);
	label1_control_config = new gcn::Label("control config scheme");
	label1_control_config->setPosition(10,70);
	label2_control_config = new gcn::Label("control config scheme");
	label2_control_config->setPosition(10,100);
	radioButton_dpad_custom = new gcn::RadioButton("Custom ", "radiodpadgroup");
	radioButton_dpad_custom->setPosition(5,10);
	radioButton_dpad_joystick = new gcn::RadioButton("Joystick ", "radiodpadgroup");
	radioButton_dpad_joystick->setPosition(5,40);
	radioButton_dpad_mouse = new gcn::RadioButton("Mouse ", "radiodpadgroup");
	radioButton_dpad_mouse->setPosition(5,70);
	radioButton_joy_port0 = new gcn::RadioButton("Port0 ", "radiojoystickgroup");
	radioButton_joy_port0->setPosition(5,10);
	radioButton_joy_port1 = new gcn::RadioButton("Port1 ", "radiojoystickgroup");
	radioButton_joy_port1->setPosition(5,40);
	radioButton_joy_both = new gcn::RadioButton("Both ", "radiojoystickgroup");
	radioButton_joy_both->setPosition(5,70);
	label1_sound = new gcn::Label("Please use Save Setting and restart application");
	label1_sound->setPosition(220,20);
	label2_sound = new gcn::Label(" after any changes in sound settings");
	label2_sound->setPosition(220,50);
	radioButton_statusln_on = new gcn::RadioButton("On ", "radiostatuslngroup");
	radioButton_statusln_on->setPosition(5,10);
	radioButton_statusln_off = new gcn::RadioButton("Off ", "radiostatuslngroup");
	radioButton_statusln_off->setPosition(5,40);
	radioButton_autofirerate_light = new gcn::RadioButton("Light ", "radioautofirerategroup");
	radioButton_autofirerate_light->setPosition(5,10);
	radioButton_autofirerate_medium = new gcn::RadioButton("Medium ", "radioautofirerategroup");
	radioButton_autofirerate_medium->setPosition(5,40);
	radioButton_autofirerate_heavy = new gcn::RadioButton("Heavy ", "radioautofirerategroup");
	radioButton_autofirerate_heavy->setPosition(5,70);
	radioButton_mouseMultiplier_25 = new gcn::RadioButton(".25 ", "radiomouseMultipliergroup");
	radioButton_mouseMultiplier_25->setPosition(5,10);
	radioButton_mouseMultiplier_50 = new gcn::RadioButton(".5 ", "radiomouseMultipliergroup");
	radioButton_mouseMultiplier_50->setPosition(50,10);
	radioButton_mouseMultiplier_1 = new gcn::RadioButton("1x ", "radiomouseMultipliergroup");
	radioButton_mouseMultiplier_1->setPosition(95,10);
	radioButton_mouseMultiplier_2 = new gcn::RadioButton("2x ", "radiomouseMultipliergroup");
	radioButton_mouseMultiplier_2->setPosition(140,10);
	radioButton_mouseMultiplier_4 = new gcn::RadioButton("4x ", "radiomouseMultipliergroup");
	radioButton_mouseMultiplier_4->setPosition(185,10);
	radioButton_tapDelay_10 = new gcn::RadioButton("normal ", "radiotapdelaygroup");
	radioButton_tapDelay_10->setPosition(5,10);
	radioButton_tapDelay_5 = new gcn::RadioButton("short ", "radiotapdelaygroup");
	radioButton_tapDelay_5->setPosition(5,40);
	radioButton_tapDelay_2 = new gcn::RadioButton("no ", "radiotapdelaygroup");
	radioButton_tapDelay_2->setPosition(5,70);
	
	check_settings();
	
	window_cpu = new gcn::Window("CPU");
	window_cpu->setPosition(10,20);
	window_cpu->add(radioButton_cpu68000);
	window_cpu->add(radioButton_cpu68020);
	window_cpu->setMovable(false);
	window_cpu->setSize(75,85);
	window_chipset = new gcn::Window("Chipset");
	window_chipset->setPosition(10,120);
	window_chipset->add(radioButton_chipsetocs);
	window_chipset->add(radioButton_chipsetecs);
	window_chipset->add(radioButton_chipsetaga);	
	window_chipset->setMovable(false);
	window_chipset->setSize(75,115);
	window_kickstart = new gcn::Window("Kickstart");
	window_kickstart->setPosition(100,20);
	window_kickstart->add(radioButton_kick12);
	window_kickstart->add(radioButton_kick13);
	window_kickstart->add(radioButton_kick20);
	window_kickstart->add(radioButton_kick31);
	window_kickstart->setMovable(false);
	window_kickstart->setSize(70,145);
	window_cpuspeed = new gcn::Window("CPU Speed");
	window_cpuspeed->setPosition(190,20);
	window_cpuspeed->add(radioButton_cpuspeed_500);
	window_cpuspeed->add(radioButton_cpuspeed_500T);
	window_cpuspeed->add(radioButton_cpuspeed_1200);
	window_cpuspeed->add(radioButton_cpuspeed_1200T);
	window_cpuspeed->add(radioButton_cpuspeed_1200T2);
	window_cpuspeed->setMovable(false);
	window_cpuspeed->setSize(85,175);
	window_floppy_number = new gcn::Window("Drives");
	window_floppy_number->setPosition(460,20);
	window_floppy_number->add(radioButton_floppy1);
	window_floppy_number->add(radioButton_floppy2);
	window_floppy_number->add(radioButton_floppy3);
	window_floppy_number->add(radioButton_floppy4);
	window_floppy_number->setMovable(false);
	window_floppy_number->setSize(55,145);
	window_floppy_speed = new gcn::Window("Speed");
	window_floppy_speed->setPosition(530,20);
	window_floppy_speed->add(radioButton_floppy_speed1x);
	window_floppy_speed->add(radioButton_floppy_speed2x);
	window_floppy_speed->add(radioButton_floppy_speed4x);
	window_floppy_speed->add(radioButton_floppy_speed8x);		
	window_floppy_speed->setMovable(false);
	window_floppy_speed->setSize(55,145);
	window_chipmem = new gcn::Window("Chip");
	window_chipmem->setPosition(20,15);
	window_chipmem->add(radioButton_chipmem_512K);
	window_chipmem->add(radioButton_chipmem_1M);
	window_chipmem->add(radioButton_chipmem_2M);
	window_chipmem->add(radioButton_chipmem_4M);
	window_chipmem->add(radioButton_chipmem_8M);
	window_chipmem->setSize(75,175);
	window_slowmem = new gcn::Window("Slow");
	window_slowmem->setPosition(110,15);
	window_slowmem->add(radioButton_slowmem_off);
	window_slowmem->add(radioButton_slowmem_512K);
	window_slowmem->add(radioButton_slowmem_1M);
	window_slowmem->add(radioButton_slowmem_15M);
	window_slowmem->setSize(75,145);
	window_fastmem = new gcn::Window("Fast");
	window_fastmem->setPosition(200,15);
	window_fastmem->add(radioButton_fastmem_off);
	window_fastmem->add(radioButton_fastmem_1M);
	window_fastmem->add(radioButton_fastmem_2M);
	window_fastmem->add(radioButton_fastmem_4M);
	window_fastmem->add(radioButton_fastmem_8M);
	window_fastmem->setSize(75,175);
	window_memory = new gcn::Window("Memory");
	window_memory->add(icon_winlogo);
	window_memory->setPosition(290,20);
	window_memory->add(window_chipmem);
	window_memory->add(window_slowmem);	
	window_memory->add(window_fastmem);
	window_memory->setMovable(false);
	window_memory->setSize(290,220);
	window_boothd = new gcn::Window("Boot HD");
	window_boothd->setPosition(10,20);
	window_boothd->add(radioButton_boothd_off);
	window_boothd->add(radioButton_boothd_hddir);
	window_boothd->add(radioButton_boothd_hdfile);	
	window_boothd->setMovable(false);
	window_boothd->setSize(70,115);
	window_width = new gcn::Window("Width");
	window_width->setPosition(10,20);
	window_width->add(radioButton_visibleAreaWidth_320);
	window_width->add(radioButton_visibleAreaWidth_640);
	window_width->add(radioButton_visibleAreaWidth_352);
	window_width->add(radioButton_visibleAreaWidth_704);
	window_width->add(radioButton_visibleAreaWidth_384);
	window_width->add(radioButton_visibleAreaWidth_768);
	window_width->setMovable(false);
	window_width->setSize(70,205);
	window_height = new gcn::Window("Height");
	window_height->setPosition(100,20);
	window_height->add(radioButton_displayedLines_200);
	window_height->add(radioButton_displayedLines_216);
	window_height->add(radioButton_displayedLines_240);
	window_height->add(radioButton_displayedLines_256);
	window_height->add(radioButton_displayedLines_262);
	window_height->add(radioButton_displayedLines_270);
	window_height->setMovable(false);
	window_height->setSize(70,205);
	window_frameskip = new gcn::Window("Frameskip");
	window_frameskip->setPosition(510,20);
	window_frameskip->add(radioButton_frameskip_0);
	window_frameskip->add(radioButton_frameskip_1);
	window_frameskip->add(radioButton_frameskip_2);
	window_frameskip->add(radioButton_frameskip_3);
	window_frameskip->add(radioButton_frameskip_4);
	window_frameskip->add(radioButton_frameskip_5);	
	window_frameskip->setMovable(false);
	window_frameskip->setSize(65,205);
	window_sound_enable = new gcn::Window("Sound");
	window_sound_enable->setPosition(10,20);
	window_sound_enable->add(radioButton_sound_off);
	window_sound_enable->add(radioButton_sound_fast);
	window_sound_enable->add(radioButton_sound_accurate);	
	window_sound_enable->setMovable(false);
	window_sound_enable->setSize(100,115);
	window_sound_rate = new gcn::Window("Rate");
	window_sound_rate->setPosition(130,20);
	window_sound_rate->add(radioButton_soundrate_8k);
	window_sound_rate->add(radioButton_soundrate_11k);
	window_sound_rate->add(radioButton_soundrate_22k);
	window_sound_rate->add(radioButton_soundrate_32k);
	window_sound_rate->add(radioButton_soundrate_44k);
	window_sound_rate->setMovable(false);
	window_sound_rate->setSize(70,175);
	window_sound_mode = new gcn::Window("Mode");
	window_sound_mode->setPosition(10,150);
	window_sound_mode->add(radioButton_soundmode_mono);
	window_sound_mode->add(radioButton_soundmode_stereo);
	window_sound_mode->setMovable(false);
	window_sound_mode->setSize(80,85);
	window_savestates_number = new gcn::Window("Number");
	window_savestates_number->setPosition(10,20);
	window_savestates_number->add(radioButton_savestate_1);
	window_savestates_number->add(radioButton_savestate_2);
	window_savestates_number->add(radioButton_savestate_3);
	window_savestates_number->add(radioButton_savestate_4);
	window_savestates_number->setMovable(false);
	window_savestates_number->setSize(60,145);
	window_savestate_screen = new gcn::Window("State screen");
	window_savestate_screen->setPosition(100,10);
	window_savestate_screen->setMovable(false);
	window_savestate_screen->setSize(320,300);
	window_savestate_screen->setVisible(false);
	window_onscreen_control = new gcn::Window("OnScreen Control");
	window_onscreen_control->setPosition(10,20);
	window_onscreen_control->add(radioButton_onscreen_show);
	window_onscreen_control->add(radioButton_onscreen_hide);
	window_onscreen_control->setMovable(false);
	window_onscreen_control->setSize(140,55);
	window_custom_control_enable = new gcn::Window("Custom Control");
	window_custom_control_enable->setPosition(10,20);
	window_custom_control_enable->add(radioButton_custom_control_off);
	window_custom_control_enable->add(radioButton_custom_control_on);
	window_custom_control_enable->setMovable(false);
	window_custom_control_enable->setSize(130,55);
	window_control_config = new gcn::Window("Control config");
	window_control_config->setPosition(10,100);
	window_control_config->add(radioButton_control_config_1);
	window_control_config->add(radioButton_control_config_2);
	window_control_config->add(radioButton_control_config_3);
	window_control_config->add(radioButton_control_config_4);
	window_control_config->add(label1_control_config);
	window_control_config->add(label2_control_config);
	window_control_config->setMovable(false);
	window_control_config->setSize(140,150);
	window_dpad = new gcn::Window("DPAD");
	window_dpad->setPosition(10,100);
	window_dpad->add(radioButton_dpad_custom);
	window_dpad->add(radioButton_dpad_joystick);
	window_dpad->add(radioButton_dpad_mouse);
	window_dpad->setMovable(false);
	window_dpad->setSize(100,115);
	window_joystick = new gcn::Window("Joystick");
	window_joystick->setPosition(160,20);
	window_joystick->add(radioButton_joy_port0);
	window_joystick->add(radioButton_joy_port1);
	window_joystick->add(radioButton_joy_both);
	window_joystick->setMovable(false);
	window_joystick->setSize(80,115);
	window_statusln = new gcn::Window("Status line");
	window_statusln->setPosition(160,150);
	window_statusln->add(radioButton_statusln_on);
	window_statusln->add(radioButton_statusln_off);
	window_statusln->setMovable(false);
	window_statusln->setSize(75,85);
	window_autofirerate = new gcn::Window("Autofire Rate");
	window_autofirerate->setPosition(250,20);
	window_autofirerate->add(radioButton_autofirerate_light);
	window_autofirerate->add(radioButton_autofirerate_medium);
	window_autofirerate->add(radioButton_autofirerate_heavy);
	window_autofirerate->setMovable(false);
	window_autofirerate->setSize(90,115);
	window_mouseMultiplier = new gcn::Window("Mouse speed");
	window_mouseMultiplier->setPosition(350,20);
	window_mouseMultiplier->add(radioButton_mouseMultiplier_25);
	window_mouseMultiplier->add(radioButton_mouseMultiplier_50);
	window_mouseMultiplier->add(radioButton_mouseMultiplier_1);
	window_mouseMultiplier->add(radioButton_mouseMultiplier_2);
	window_mouseMultiplier->add(radioButton_mouseMultiplier_4);
	window_mouseMultiplier->setMovable(false);
	window_mouseMultiplier->setSize(240,55);
	window_tapDelay = new gcn::Window("Tap delay");
	window_tapDelay->setPosition(350,100);
	window_tapDelay->add(radioButton_tapDelay_10);
	window_tapDelay->add(radioButton_tapDelay_5);
	window_tapDelay->add(radioButton_tapDelay_2);
	window_tapDelay->setMovable(false);
	window_tapDelay->setSize(90,115);
	
	window_main = new gcn::Window("Main");
	window_main->add(icon_winlogo);
	window_main->add(window_cpu);
	window_main->add(window_chipset);
	window_main->add(window_kickstart);
	window_main->add(window_cpuspeed);
	window_main->add(window_memory);
	window_main->setMovable(false);
	window_main->setSize(600,275);
	window_floppy = new gcn::Window("Floppy drives");
	window_floppy->add(icon_winlogo);
	window_floppy->add(window_floppy_number);
	window_floppy->add(window_floppy_speed);
	window_floppy->add(button_df0);
	window_floppy->add(textField_df0);
	window_floppy->add(button_df1);  
	window_floppy->add(textField_df1);
	window_floppy->add(button_df2);  
	window_floppy->add(textField_df2);
	window_floppy->add(button_df3);  
	window_floppy->add(textField_df3);
	window_floppy->add(button_ejectallfloppy);
	window_floppy->add(button_save_config_for_game);
	window_floppy->setMovable(false);
	window_floppy->setSize(600,275);
	window_hard = new gcn::Window("Hard drives");
	window_hard->add(icon_winlogo);
	window_hard->add(window_boothd);
	window_hard->add(button_hddir);
	window_hard->add(textField_hddir);
	window_hard->add(button_hdfile);  
	window_hard->add(textField_hdfile);
	window_hard->add(button_save_config_for_hd);
	window_hard->setMovable(false);
	window_hard->setSize(600,275);
	window_display = new gcn::Window("Display");
	window_display->add(icon_winlogo);
	window_display->add(window_width);
	window_display->add(window_height);
	window_display->add(window_frameskip);
	window_display->setMovable(false);
	window_display->setSize(600,275);
	window_sound = new gcn::Window("Sound");
	window_sound->add(icon_winlogo);
	window_sound->add(window_sound_enable);
	window_sound->add(window_sound_rate);
	window_sound->add(window_sound_mode);
	window_sound->add(label1_sound);
	window_sound->add(label2_sound);
	window_sound->setMovable(false);
	window_sound->setSize(600,275);
	window_savestates = new gcn::Window("Savestates");
	window_savestates->add(icon_winlogo);
	if (emulating)
	{
	  window_savestates->add(window_savestates_number);
	  window_savestates->add(button_load_savestate);
	  window_savestates->add(button_save_savestate);
	  window_savestates->add(window_savestate_screen);
	}
	 else
	   window_savestates->add(label_ss_error);
	window_savestates->setMovable(false);
	window_savestates->setSize(600,275);
	window_control = new gcn::Window("Control");
	window_control->add(icon_winlogo);
	window_control->add(window_onscreen_control);
	window_control->add(window_control_config);
	window_control->add(window_joystick);
	window_control->add(window_statusln);
	window_control->add(window_autofirerate);
	window_control->add(window_mouseMultiplier);
	window_control->add(window_tapDelay);
	window_control->setMovable(false);
	window_control->setSize(600,275);
	window_custom_control = new gcn::Window("Custom control");
	window_custom_control->add(icon_winlogo);
	window_custom_control->add(window_custom_control_enable);
	window_custom_control->add(window_dpad);
	window_custom_control->setMovable(false);
	window_custom_control->setSize(600,275);
	tabbedArea->addTab("  Main  ", window_main);
        tabTwoCheckBox = new gcn::CheckBox("A check box in tab 2");
        tabbedArea->addTab(" Floppy drives ", window_floppy);
        tabbedArea->addTab(" Hard drives ", window_hard);
        tabbedArea->addTab(" Display ", window_display);
        tabbedArea->addTab(" Sound ", window_sound);
        tabbedArea->addTab(" Savestates ", window_savestates);
        tabbedArea->addTab(" Control ", window_control);
        tabbedArea->addTab(" Custom Control ", window_custom_control);

	button_presetA500 = new gcn::Button("A500");
	button_presetA500->setPosition(20,10);
	button_presetA1200 = new gcn::Button("A1200");
	button_presetA1200->setPosition(18,50);
	window_preset = new gcn::Window("Presets");
	window_preset->add(button_presetA500);
        window_preset->add(button_presetA1200);
	window_preset->setMovable(false);
	window_preset->setSize(80,110);
//        window_preset->resizeToContent();

        image_logo = gcn::Image::load("data/logo_uae4all.jpg");
        icon_logo = new gcn::Icon(image_logo);	
	
	button_reset = new gcn::Button("Start");
        button_reset->setWidth(90);
        button_reset->setHeight(50);
	button_resume = new gcn::Button("Resume");
        button_resume->setWidth(90);
        button_resume->setHeight(50);
	button_quit = new gcn::Button("Quit");
        button_quit->setWidth(90);
        button_quit->setHeight(50);
	button_save_config = new gcn::Button("Save Config");
        button_save_config->setWidth(90);
        button_save_config->setHeight(40);
	
        // Now it's time to add the widgets to the top container
        // so they will be conected to the GUI.
        
/*       top->add(label, 10, 10);
        top->add(icon, 10, 30);
        top->add(button, 200, 10);
        top->add(textField, 250, 10);
        top->add(textBoxScrollArea, 200, 50);
        top->add(listBox, 200, 200);
        top->add(dropDown, 500, 10);
        top->add(checkBox1, 500, 130);
        top->add(checkBox2, 500, 150);
        top->add(slider, 500, 300);
        top->add(window, 50, 350);
        */
        top->add(icon, 0, 0);
        top->add(tabbedArea, 20, 20);
        top->add(window_preset, 550, 350);
        top->add(icon_logo, 20, 340);
        top->add(button_reset, 200, 380);
        top->add(button_resume, 320, 380);
        top->add(button_quit, 430, 410);
        top->add(button_save_config, 430, 350);
	
        // Now we set the button's action event ids.
        button_reset->setActionEventId("button_reset");
        button_resume->setActionEventId("button_resume");
        button_quit->setActionEventId("button_quit");
	button_save_config->setActionEventId("Save Config");

        // We make an instance of the ButtonActionListener.
        buttonActionListener = new ButtonActionListener();

        // And add the ButtonActionListener as an action listener of the buttons.
        button_reset->addActionListener(buttonActionListener);
        button_resume->addActionListener(buttonActionListener);
        button_quit->addActionListener(buttonActionListener);
	button_save_config->addActionListener(buttonActionListener);
	button_ok->addActionListener(buttonActionListener);
	button_cancel->addActionListener(buttonActionListener);
	button_select->addActionListener(buttonActionListener);
	button_open->addActionListener(buttonActionListener);	
	listBox->addActionListener(buttonActionListener);
	button_presetA500->addActionListener(buttonActionListener);
	button_presetA1200->addActionListener(buttonActionListener);
	radioButton_cpu68000->addActionListener(buttonActionListener);
	radioButton_cpu68020->addActionListener(buttonActionListener);
	radioButton_chipsetocs->addActionListener(buttonActionListener);
	radioButton_chipsetecs->addActionListener(buttonActionListener);
	radioButton_chipsetaga->addActionListener(buttonActionListener);
	radioButton_kick12->addActionListener(buttonActionListener);
	radioButton_kick13->addActionListener(buttonActionListener);
	radioButton_kick20->addActionListener(buttonActionListener);
	radioButton_kick31->addActionListener(buttonActionListener);
	radioButton_cpuspeed_500->addActionListener(buttonActionListener);
	radioButton_cpuspeed_500T->addActionListener(buttonActionListener);
	radioButton_cpuspeed_1200->addActionListener(buttonActionListener);
	radioButton_cpuspeed_1200T->addActionListener(buttonActionListener);
	radioButton_cpuspeed_1200T2->addActionListener(buttonActionListener);
	radioButton_floppy1->addActionListener(buttonActionListener);
	radioButton_floppy2->addActionListener(buttonActionListener);
	radioButton_floppy3->addActionListener(buttonActionListener);
	radioButton_floppy4->addActionListener(buttonActionListener);
	button_save_config_for_game->addActionListener(buttonActionListener);
	button_ejectallfloppy->addActionListener(buttonActionListener);
	radioButton_floppy_speed1x->addActionListener(buttonActionListener);
	radioButton_floppy_speed2x->addActionListener(buttonActionListener);
	radioButton_floppy_speed4x->addActionListener(buttonActionListener);
	radioButton_floppy_speed8x->addActionListener(buttonActionListener);
	button_df0->addActionListener(buttonActionListener);
	button_df1->addActionListener(buttonActionListener);
	button_df2->addActionListener(buttonActionListener);
	button_df3->addActionListener(buttonActionListener);
	radioButton_chipmem_512K->addActionListener(buttonActionListener);
	radioButton_chipmem_1M->addActionListener(buttonActionListener);
	radioButton_chipmem_2M->addActionListener(buttonActionListener);
	radioButton_chipmem_4M->addActionListener(buttonActionListener);
	radioButton_chipmem_8M->addActionListener(buttonActionListener);
	radioButton_slowmem_off->addActionListener(buttonActionListener);
	radioButton_slowmem_512K->addActionListener(buttonActionListener);
	radioButton_slowmem_1M->addActionListener(buttonActionListener);
	radioButton_slowmem_15M->addActionListener(buttonActionListener);
	radioButton_fastmem_off->addActionListener(buttonActionListener);
	radioButton_fastmem_1M->addActionListener(buttonActionListener);
	radioButton_fastmem_2M->addActionListener(buttonActionListener);
	radioButton_fastmem_4M->addActionListener(buttonActionListener);
	radioButton_fastmem_8M->addActionListener(buttonActionListener);
	radioButton_boothd_off->addActionListener(buttonActionListener);
	radioButton_boothd_hddir->addActionListener(buttonActionListener);
	radioButton_boothd_hdfile->addActionListener(buttonActionListener);
	button_hddir->addActionListener(buttonActionListener);
	button_hdfile->addActionListener(buttonActionListener);
	button_save_config_for_hd->addActionListener(buttonActionListener);
	radioButton_visibleAreaWidth_320->addActionListener(buttonActionListener);
	radioButton_visibleAreaWidth_640->addActionListener(buttonActionListener);
	radioButton_visibleAreaWidth_352->addActionListener(buttonActionListener);
	radioButton_visibleAreaWidth_704->addActionListener(buttonActionListener);
	radioButton_visibleAreaWidth_384->addActionListener(buttonActionListener);
	radioButton_visibleAreaWidth_768->addActionListener(buttonActionListener);
	radioButton_displayedLines_200->addActionListener(buttonActionListener);
	radioButton_displayedLines_216->addActionListener(buttonActionListener);
	radioButton_displayedLines_240->addActionListener(buttonActionListener);
	radioButton_displayedLines_256->addActionListener(buttonActionListener);
	radioButton_displayedLines_262->addActionListener(buttonActionListener);
	radioButton_displayedLines_270->addActionListener(buttonActionListener);
	radioButton_frameskip_0->addActionListener(buttonActionListener);
	radioButton_frameskip_1->addActionListener(buttonActionListener);
	radioButton_frameskip_2->addActionListener(buttonActionListener);
	radioButton_frameskip_3->addActionListener(buttonActionListener);
	radioButton_frameskip_4->addActionListener(buttonActionListener);
	radioButton_frameskip_5->addActionListener(buttonActionListener);
	radioButton_sound_off->addActionListener(buttonActionListener);
	radioButton_sound_fast->addActionListener(buttonActionListener);
	radioButton_sound_accurate->addActionListener(buttonActionListener);
	radioButton_soundrate_8k->addActionListener(buttonActionListener);
	radioButton_soundrate_11k->addActionListener(buttonActionListener);
	radioButton_soundrate_22k->addActionListener(buttonActionListener);
	radioButton_soundrate_32k->addActionListener(buttonActionListener);
	radioButton_soundrate_44k->addActionListener(buttonActionListener);
	radioButton_soundmode_mono->addActionListener(buttonActionListener);
	radioButton_soundmode_stereo->addActionListener(buttonActionListener);
	radioButton_savestate_1->addActionListener(buttonActionListener);
	radioButton_savestate_2->addActionListener(buttonActionListener);
	radioButton_savestate_3->addActionListener(buttonActionListener);
	radioButton_savestate_4->addActionListener(buttonActionListener);
	button_load_savestate->addActionListener(buttonActionListener);
	button_save_savestate->addActionListener(buttonActionListener);
	button_warning->addActionListener(buttonActionListener);
	radioButton_onscreen_hide->addActionListener(buttonActionListener);
	radioButton_onscreen_show->addActionListener(buttonActionListener);
	radioButton_custom_control_off->addActionListener(buttonActionListener);
	radioButton_custom_control_on->addActionListener(buttonActionListener);
	radioButton_control_config_1->addActionListener(buttonActionListener);
	radioButton_control_config_2->addActionListener(buttonActionListener);
	radioButton_control_config_3->addActionListener(buttonActionListener);
	radioButton_control_config_4->addActionListener(buttonActionListener);
	radioButton_dpad_custom->addActionListener(buttonActionListener);
	radioButton_dpad_joystick->addActionListener(buttonActionListener);
	radioButton_dpad_mouse->addActionListener(buttonActionListener);
	radioButton_joy_port0->addActionListener(buttonActionListener);
	radioButton_joy_port1->addActionListener(buttonActionListener);
	radioButton_joy_both->addActionListener(buttonActionListener);
	radioButton_statusln_off->addActionListener(buttonActionListener);
	radioButton_statusln_on->addActionListener(buttonActionListener);
	radioButton_autofirerate_light->addActionListener(buttonActionListener);
	radioButton_autofirerate_medium->addActionListener(buttonActionListener);
	radioButton_autofirerate_heavy->addActionListener(buttonActionListener);
	radioButton_mouseMultiplier_25->addActionListener(buttonActionListener);
	radioButton_mouseMultiplier_50->addActionListener(buttonActionListener);
	radioButton_mouseMultiplier_1->addActionListener(buttonActionListener);
	radioButton_mouseMultiplier_2->addActionListener(buttonActionListener);
	radioButton_mouseMultiplier_4->addActionListener(buttonActionListener);
	radioButton_tapDelay_10->addActionListener(buttonActionListener);
	radioButton_tapDelay_5->addActionListener(buttonActionListener);
	radioButton_tapDelay_2->addActionListener(buttonActionListener);
    }

    void check_settings()
    {
      	static char bufname1[128];
	static char bufname2[128];
	static char bufname3[128];
	static char bufname4[128];
	static char bufname5[128];
	static char bufname6[128];
	if (mainMenu_CPU_model==0)
	      radioButton_cpu68000->setSelected(true);
	else if (mainMenu_CPU_model==1)
	      radioButton_cpu68020->setSelected(true);
	if (mainMenu_chipset==0)
	      radioButton_chipsetocs->setSelected(true);
	else if (mainMenu_chipset==1)
	      radioButton_chipsetecs->setSelected(true);
	else if (mainMenu_chipset==2)
	      radioButton_chipsetaga->setSelected(true);
	if (kickstart==0)
	      radioButton_kick12->setSelected(true);
	else if (kickstart==1)
	      radioButton_kick13->setSelected(true);
	else if (kickstart==2)
	      radioButton_kick20->setSelected(true);
	else if (kickstart==3)
	      radioButton_kick31->setSelected(true);
	if (mainMenu_CPU_speed==0)
	      radioButton_cpuspeed_500->setSelected(true);
	else if (mainMenu_CPU_speed==1)
	      radioButton_cpuspeed_500T->setSelected(true);
	else if (mainMenu_CPU_speed==2)
	      radioButton_cpuspeed_1200->setSelected(true);
	else if (mainMenu_CPU_speed==3)
	      radioButton_cpuspeed_1200T->setSelected(true);
	else if (mainMenu_CPU_speed==4)
	      radioButton_cpuspeed_1200T2->setSelected(true);
	if (nr_drives==1)
	      radioButton_floppy1->setSelected(true);
	else if (nr_drives==2)
	      radioButton_floppy2->setSelected(true);
	else if (nr_drives==3)
	      radioButton_floppy3->setSelected(true);
	else if (nr_drives==4)
	      radioButton_floppy4->setSelected(true);
	
	if(strcmp(uae4all_image_file0, "")==0)
	  textField_df0->setText("insert disk image");
	else
	{
	  extractFileName(uae4all_image_file0, bufname1);
	  textField_df0->setText(bufname1);
	}

	if (nr_drives<4)
	      textField_df3->setText("disabled");
	if (nr_drives<3)
	      textField_df2->setText("disabled");
	if (nr_drives<2)
	      textField_df1->setText("disabled");
	if (nr_drives>1)
	{
	      if (strcmp(uae4all_image_file1, "")==0)
		textField_df1->setText("insert disk image");
	      else
	      {
	      extractFileName(uae4all_image_file1, bufname2);
	      textField_df1->setText(bufname2);
	      }
	}
	if (nr_drives>2)
	{
	      if (strcmp(uae4all_image_file2, "")==0)
		textField_df2->setText("insert disk image");
	      else
	      {
	      extractFileName(uae4all_image_file2, bufname3);
	      textField_df2->setText(bufname3);
	      }
	}
	if (nr_drives>3)
	{
	      if (strcmp(uae4all_image_file3, "")==0)
		textField_df3->setText("insert disk image");
	      else
	      {
	      extractFileName(uae4all_image_file3, bufname4);
	      textField_df3->setText(bufname4);
	      }
	}
	if (mainMenu_floppyspeed==100)
	      radioButton_floppy_speed1x->setSelected(true);
	else if (mainMenu_floppyspeed==200)
	      radioButton_floppy_speed2x->setSelected(true);
	else if (mainMenu_floppyspeed==400)
	      radioButton_floppy_speed4x->setSelected(true);
	else if (mainMenu_floppyspeed==800)
	      radioButton_floppy_speed8x->setSelected(true);
	if (mainMenu_chipMemory==0)
	      radioButton_chipmem_512K->setSelected(true);
	else if (mainMenu_chipMemory==1)
	      radioButton_chipmem_1M->setSelected(true);
	else if (mainMenu_chipMemory==2)
	      radioButton_chipmem_2M->setSelected(true);
	else if (mainMenu_chipMemory==3)
	      radioButton_chipmem_4M->setSelected(true);
	else if (mainMenu_chipMemory==4)
	      radioButton_chipmem_8M->setSelected(true);
	if (mainMenu_slowMemory==0)
	      radioButton_slowmem_off->setSelected(true);
	else if (mainMenu_slowMemory==1)
	      radioButton_slowmem_512K->setSelected(true);
	else if (mainMenu_slowMemory==2)
	      radioButton_slowmem_1M->setSelected(true);
	else if (mainMenu_slowMemory==3)
	      radioButton_slowmem_15M->setSelected(true);
	if (mainMenu_fastMemory==0)
	      radioButton_fastmem_off->setSelected(true);
	else if (mainMenu_fastMemory==1)
	      radioButton_fastmem_1M->setSelected(true);
	else if (mainMenu_fastMemory==2)
	      radioButton_fastmem_2M->setSelected(true);
	else if (mainMenu_fastMemory==3)
	      radioButton_fastmem_4M->setSelected(true);
	else if (mainMenu_fastMemory==4)
	      radioButton_fastmem_8M->setSelected(true);
	if (mainMenu_bootHD==0)
	      radioButton_boothd_off->setSelected(true);
	else if (mainMenu_bootHD==1)
	      radioButton_boothd_hddir->setSelected(true);
	else if (mainMenu_bootHD==2)
	      radioButton_boothd_hdfile->setSelected(true);
	if (strcmp(uae4all_hard_dir, "")==0)
	  textField_hddir->setText("not selected");
	else
	{
	  extractFileName(uae4all_hard_dir, bufname5);
	  textField_hddir->setText(bufname5);
	}
	if (strcmp(uae4all_hard_file, "")==0)
	  textField_hdfile->setText("not selected");
	else
	{
	  extractFileName(uae4all_hard_file, bufname6);
	  textField_hdfile->setText(bufname6);
	}
	if (visibleAreaWidth==320)
	      radioButton_visibleAreaWidth_320->setSelected(true);
	else if (visibleAreaWidth==640)
	      radioButton_visibleAreaWidth_640->setSelected(true);
	else if (visibleAreaWidth==352)
	      radioButton_visibleAreaWidth_352->setSelected(true);
	else if (visibleAreaWidth==704)
	      radioButton_visibleAreaWidth_704->setSelected(true);
	else if (visibleAreaWidth==384)
	      radioButton_visibleAreaWidth_384->setSelected(true);
	else if (visibleAreaWidth==768)
	      radioButton_visibleAreaWidth_768->setSelected(true);
	if (mainMenu_displayedLines==200)
	      radioButton_displayedLines_200->setSelected(true);
	else if (mainMenu_displayedLines==216)
	      radioButton_displayedLines_216->setSelected(true);
	else if (mainMenu_displayedLines==240)
	      radioButton_displayedLines_240->setSelected(true);
	else if (mainMenu_displayedLines==256)
	      radioButton_displayedLines_256->setSelected(true);
	else if (mainMenu_displayedLines==262)
	      radioButton_displayedLines_262->setSelected(true);
	else if (mainMenu_displayedLines==270)
	      radioButton_displayedLines_270->setSelected(true);
	if (mainMenu_frameskip==0)
	      radioButton_frameskip_0->setSelected(true);
	else if (mainMenu_frameskip==1)
	      radioButton_frameskip_1->setSelected(true);
	else if (mainMenu_frameskip==2)
	      radioButton_frameskip_2->setSelected(true);
	else if (mainMenu_frameskip==3)
	      radioButton_frameskip_3->setSelected(true);
	else if (mainMenu_frameskip==4)
	      radioButton_frameskip_4->setSelected(true);
	else if (mainMenu_frameskip==5)
	      radioButton_frameskip_5->setSelected(true);
	if (mainMenu_sound==0)
	      radioButton_sound_off->setSelected(true);
	else if (mainMenu_sound==1)
	      radioButton_sound_fast->setSelected(true);
	else if (mainMenu_sound==2)
	      radioButton_sound_accurate->setSelected(true);
	if (sound_rate==8000)
	      radioButton_soundrate_8k->setSelected(true);
	else if (sound_rate==11025)
	      radioButton_soundrate_11k->setSelected(true);
	else if (sound_rate==22050)
	      radioButton_soundrate_22k->setSelected(true);
	else if (sound_rate==32000)
	      radioButton_soundrate_32k->setSelected(true);
	else if (sound_rate==44100)
	      radioButton_soundrate_44k->setSelected(true);
	if (mainMenu_soundStereo==0)
	      radioButton_soundmode_mono->setSelected(true);
	else if (mainMenu_soundStereo==1)
	      radioButton_soundmode_stereo->setSelected(true);
	if (saveMenu_n_savestate==0)
	      radioButton_savestate_1->setSelected(true);
	else if (saveMenu_n_savestate==1)
	      radioButton_savestate_2->setSelected(true);
	else if (saveMenu_n_savestate==2)
	      radioButton_savestate_3->setSelected(true);
	else if (saveMenu_n_savestate==3)
	      radioButton_savestate_4->setSelected(true);
	if (mainMenu_onScreen==0)
	      radioButton_onscreen_hide->setSelected(true);
	else if (mainMenu_onScreen==1)
	      radioButton_onscreen_show->setSelected(true);
	if (mainMenu_customControls==1)
	      radioButton_custom_control_on->setSelected(true);
	else if (mainMenu_customControls==0)
	      radioButton_custom_control_off->setSelected(true);
	if (mainMenu_joyConf==0)
	{
	      radioButton_control_config_1->setSelected(true);
	      label1_control_config->setCaption("A=Autofire X=Fire");
	      label2_control_config->setCaption("Y=Space B=2nd");
	}
	else if (mainMenu_joyConf==1)
	{
	      radioButton_control_config_2->setSelected(true);
	      label1_control_config->setCaption("A=Fire X=Autofire");
	      label2_control_config->setCaption("Y=Space B=2nd");
	}
	else if (mainMenu_joyConf==2)
	{
	      radioButton_control_config_3->setSelected(true);
	      label1_control_config->setCaption("A=Autofire X=Jump");
	      label2_control_config->setCaption("Y=Fire B=2nd");
	}
	else if (mainMenu_joyConf==3)
	{
	      radioButton_control_config_4->setSelected(true);
	      label1_control_config->setCaption("A=Fire X=Jump");
	      label2_control_config->setCaption("Y=Autofire B=2nd");
	}
	if (mainMenu_custom_dpad==0)
	      radioButton_dpad_custom->setSelected(true);
	else if (mainMenu_custom_dpad==1)
	      radioButton_dpad_joystick->setSelected(true);
	else if (mainMenu_custom_dpad==2)
	      radioButton_dpad_mouse->setSelected(true);
	if (mainMenu_joyPort==1)
	      radioButton_joy_port0->setSelected(true);
	else if (mainMenu_joyPort==2)
	      radioButton_joy_port1->setSelected(true);
	else if (mainMenu_joyPort==0)
	      radioButton_joy_both->setSelected(true);
	if (mainMenu_showStatus)
	      radioButton_statusln_on->setSelected(true);
	else if (!mainMenu_showStatus)
	      radioButton_statusln_off->setSelected(true);
	if (mainMenu_autofireRate==12)
	      radioButton_autofirerate_light->setSelected(true);
	else if (mainMenu_autofireRate==8)
	      radioButton_autofirerate_medium->setSelected(true);
	else if (mainMenu_autofireRate==4)
	      radioButton_autofirerate_heavy->setSelected(true);
	if (mainMenu_mouseMultiplier==25)
	      radioButton_mouseMultiplier_25->setSelected(true);
	else if (mainMenu_mouseMultiplier==50)
	      radioButton_mouseMultiplier_50->setSelected(true);
	else if (mainMenu_mouseMultiplier==1)
	      radioButton_mouseMultiplier_1->setSelected(true);
	else if (mainMenu_mouseMultiplier==2)
	      radioButton_mouseMultiplier_2->setSelected(true);
	else if (mainMenu_mouseMultiplier==4)
	      radioButton_mouseMultiplier_4->setSelected(true);
	if (mainMenu_tapDelay==10)
	      radioButton_tapDelay_10->setSelected(true);
	else if (mainMenu_tapDelay==5)
	      radioButton_tapDelay_5->setSelected(true);
	else if (mainMenu_tapDelay==2)
	      radioButton_tapDelay_2->setSelected(true);

check_savestate_screen();	

}

void showWarning(const char *msg)
{
	label_text->setCaption(msg);
	window_warning->add(label_text);
        top->add(window_warning, 240, 220);
}
void show_error(const char *msg)
{
	label_text->setCaption(msg);
	window_warning->add(label_text);
        top->add(window_warning, 240, 220);
}

#include <sys/mman.h>
#include <sys/types.h>
#include <sys/ioctl.h>
#include <sys/stat.h>
#include <unistd.h>
#include <dirent.h>
#include "uae.h"
#include "gp2xutil.h"

#ifndef PATH_MAX
	#define PATH_MAX 256
#endif

char romFileName[PATH_MAX];

/* What is being loaded, floppy/hd dir/hdf */

char filename0[256] = "";
char filename1[256] = "";
char filename2[256] = "";
char filename3[256] = "";

int text_dir_num_files=0, text_dir_num_files_index=0;

static void extractFileName(char * str,char *buffer)
{
	char *p=str+strlen(str)-1;
	while(*p != '/') p--;
	p++;
	strcpy(buffer,p);
}

void check_presetModeId()
{
  if (visibleAreaWidth==320 && mainMenu_displayedLines == 200)
    presetModeId=0;
  else if (visibleAreaWidth==320 && mainMenu_displayedLines == 216)
    presetModeId=1;
  else if (visibleAreaWidth==320 && mainMenu_displayedLines == 240)
    presetModeId=2;
  else if (visibleAreaWidth==320 && mainMenu_displayedLines == 256)
    presetModeId=3;
  else if (visibleAreaWidth==320 && mainMenu_displayedLines == 262)
    presetModeId=4;
  else if (visibleAreaWidth==320 && mainMenu_displayedLines == 270)
    presetModeId=5;
  else if (visibleAreaWidth==640 && mainMenu_displayedLines == 200)
    presetModeId=10;
  else if (visibleAreaWidth==640 && mainMenu_displayedLines == 216)
    presetModeId=11;
  else if (visibleAreaWidth==640 && mainMenu_displayedLines == 240)
    presetModeId=12;
  else if (visibleAreaWidth==640 && mainMenu_displayedLines == 256)
    presetModeId=13;
  else if (visibleAreaWidth==640 && mainMenu_displayedLines == 262)
    presetModeId=14;
  else if (visibleAreaWidth==640 && mainMenu_displayedLines == 270)
    presetModeId=15;  
}

void check_savestate_screen()
{
	if (emulating && tabbedArea->getSelectedTabIndex()==5)
	{
	      	strcpy(screenshot_filename, uae4all_image_file0);
			switch(saveMenu_n_savestate)
			{
				case 1:
					strcat(screenshot_filename,"-1.png"); break;
				case 2:
					strcat(screenshot_filename,"-2.png"); break;
				case 3:
					strcat(screenshot_filename,"-3.png"); break;
				default: 
					strcat(screenshot_filename,".png");
			}
			FILE *f=fopen(screenshot_filename,"rb");
			if (f)
			{
				fclose(f);
				image_savestate = gcn::Image::load(screenshot_filename);
				icon_savestate = new gcn::Icon(image_savestate);
				window_savestate_screen->setVisible(true);
				window_savestate_screen->add(icon_savestate);
				image_savestate = NULL;
				icon_savestate = NULL;
			}
			else
			{
				window_savestate_screen->setVisible(false);
			}
	}
}

void checkfilename (char *currentfilename)
{
	char *ret = NULL, *fname = NULL;
	char *ptr;
	char actualpath [PATH_MAX];
	struct dirent **namelist;
	DIR *dir;
	
	if ((dir = opendir(currentfilename)))
	{ 
	  dirList=currentfilename;
	  ptr = realpath(currentfilename, actualpath);
	  strcpy(currentDir, ptr);
	  closedir(dir);
	}
	else 
	{
	if (menu_load_type == MENU_LOAD_HDF)
	{
		strcpy(uae4all_hard_file, currentfilename);
		make_hard_file_cfg_line(uae4all_hard_file);
		reset_hdConf();
		mainMenu_bootHD = 2;
		loadconfig(2);
	}
	else if (current_drive==0)
	{
		strcpy(uae4all_image_file0, currentfilename);
		// Check for disk-specific config
		char path[300];
		create_configfilename(path, uae4all_image_file0, 0);
		FILE *f=fopen(path,"rt");
		if(f)
		{
			// config file exists -> load
			fclose(f);
			loadconfig();
		}
	}
	else if (current_drive==1)
		strcpy(uae4all_image_file1, currentfilename);
	else if (current_drive==2)
		strcpy(uae4all_image_file2, currentfilename);
	else if (current_drive==3)
		strcpy(uae4all_image_file3, currentfilename);

	  unraise_loadMenu2();
	}
}

static int menuLoadLoop(char *curr_path)
{
	char *ret = NULL, *fname = NULL;
	DIR *dir;

	// is this a dir or a full path?
	if ((dir = opendir(curr_path)))
		closedir(dir);
	else 
	{
		char *p;
		for (p = curr_path + strlen(curr_path) - 1; p > curr_path && *p != '/'; p--);
		*p = 0;
		fname = p+1;
	}
	dirList=curr_path;

	if (menu_load_type == MENU_LOAD_HD_DIR)
		window_load->setCaption("  Select HD-dir  ");
	else if (menu_load_type == MENU_LOAD_HDF)
		window_load->setCaption("       Select .HDF-file       ");
	else if (current_drive==0)
		window_load->setCaption(" Insert .ADF or .ADZ into DF0 ");
	else if (current_drive==1)
		window_load->setCaption(" Insert .ADF or .ADZ into DF1 ");
	else if (current_drive==2)
		window_load->setCaption(" Insert .ADF or .ADZ into DF2 ");
	else if (current_drive==3)
		window_load->setCaption(" Insert .ADF or .ADZ into DF3 ");
	else
		window_load->setCaption("File Manager");
	
}

static void raise_loadMenu2()
{
	if (menu_load_type != MENU_LOAD_HD_DIR)
	  window_load->add(button_ok);
	else
	{
	  window_load->add(button_select);
	  window_load->add(button_open);
	}
        top->add(window_load, 180, 150);
}

static void unraise_loadMenu2()
{
	if (menu_load_type != MENU_LOAD_HD_DIR)
	  window_load->remove(button_ok);
	else
	{
	  window_load->remove(button_select);
	  window_load->remove(button_open);
	} 
        top->remove(window_load);
}

void run_menuLoad_guichan(char *curr_path, int aLoadType)
{
	menu_load_type = aLoadType;
	raise_loadMenu2();
	int ret=menuLoadLoop(curr_path);
}
    
    /**
     * Halts the widgets example.
     */
    void halt()
    {
        delete font;
        delete top;
        delete label;
        delete icon;
        delete button;
        delete textField;
        delete textBox;
        delete textBoxScrollArea;
        delete listBox;
        delete listBoxScrollArea;
        delete dropDown;
        delete checkBox1;
        delete checkBox2;
        delete button_presetA500;
        delete button_presetA1200;
        delete radioButton_cpu68000;
        delete radioButton_cpu68020;
        delete radioButton_chipsetocs;
        delete radioButton_chipsetecs;
        delete radioButton_chipsetaga;
        delete radioButton_kick12;
        delete radioButton_kick13;
        delete radioButton_kick20;
        delete radioButton_kick31;
        delete radioButton_cpuspeed_500;
        delete radioButton_cpuspeed_500T;
        delete radioButton_cpuspeed_1200;
        delete radioButton_cpuspeed_1200T;
        delete radioButton_cpuspeed_1200T2; 
        delete slider;
        delete window;
        delete tabbedArea;
        delete tabOneButton;
        delete tabTwoCheckBox;
        delete window_main;
        delete window_floppy;
        delete window_hard;
        delete window_preset;
        delete window_cpu;
        delete window_chipset;
        delete window_kickstart;
        delete window_cpuspeed;
        delete button_reset;
        delete button_resume;
        delete button_quit;
        delete button_save_config;
        delete button_save_config_for_game;
        delete icon_logo;
	delete window_warning;
	delete label_text;
	delete window_floppy_number;    
	delete radioButton_floppy1;
	delete radioButton_floppy2;
	delete radioButton_floppy3;
	delete radioButton_floppy4;
	delete button_ejectallfloppy;
	delete button_df0;
	delete button_df1;
	delete button_df2;
	delete button_df3;
	delete textField_df0;
	delete textField_df1;
	delete textField_df2;
	delete textField_df3;
	delete window_floppy_speed;
	delete radioButton_floppy_speed1x;
	delete radioButton_floppy_speed2x;
	delete radioButton_floppy_speed4x;
	delete radioButton_floppy_speed8x;
	delete window_load;
	delete button_ok;
	delete button_cancel;
	delete radioButton_chipmem_512K;
	delete radioButton_chipmem_1M;
	delete radioButton_chipmem_2M;
	delete radioButton_chipmem_4M;
	delete radioButton_chipmem_8M;
	delete radioButton_slowmem_off;
	delete radioButton_slowmem_512K;
	delete radioButton_slowmem_1M;
	delete radioButton_slowmem_15M;
	delete radioButton_fastmem_off;
	delete radioButton_fastmem_1M;
	delete radioButton_fastmem_2M;
	delete radioButton_fastmem_4M;
	delete radioButton_fastmem_8M;
	delete window_memory;
	delete window_chipmem;
	delete window_slowmem;
	delete window_fastmem;
	delete window_boothd;
	delete radioButton_boothd_off;
	delete radioButton_boothd_hddir;
	delete radioButton_boothd_hdfile;
	delete button_hddir;
	delete button_hdfile;
	delete textField_hddir;
	delete textField_hdfile;
	delete button_save_config_for_hd;
	delete icon_winlogo;
	delete image_winlogo;
	delete window_display;
	delete window_sound;
	delete window_savestates;
	delete window_control;
	delete window_custom_control;
	delete radioButton_visibleAreaWidth_320;
	delete radioButton_visibleAreaWidth_640;
	delete radioButton_visibleAreaWidth_352;
	delete radioButton_visibleAreaWidth_704;
	delete radioButton_visibleAreaWidth_384;
	delete radioButton_visibleAreaWidth_768;
	delete radioButton_displayedLines_200;
	delete radioButton_displayedLines_216;
	delete radioButton_displayedLines_240;
	delete radioButton_displayedLines_256;
	delete radioButton_displayedLines_262;
	delete radioButton_displayedLines_270;
	delete radioButton_frameskip_0;
	delete radioButton_frameskip_1;
	delete radioButton_frameskip_2;
	delete radioButton_frameskip_3;
	delete radioButton_frameskip_4;
	delete radioButton_frameskip_5;
	delete window_width;
	delete window_height;
	delete window_frameskip;
	delete radioButton_sound_off;
	delete radioButton_sound_fast;
	delete radioButton_sound_accurate;
	delete radioButton_soundrate_8k;
	delete radioButton_soundrate_11k;
	delete radioButton_soundrate_22k;
	delete radioButton_soundrate_32k;
	delete radioButton_soundrate_44k;
	delete radioButton_soundmode_mono;
	delete radioButton_soundmode_stereo;
	delete window_sound_enable;
	delete window_sound_rate;
	delete window_sound_mode;
	delete radioButton_savestate_1;
	delete radioButton_savestate_2;
	delete radioButton_savestate_3;
	delete radioButton_savestate_4;
	delete button_load_savestate;
	delete button_save_savestate;
	delete window_savestates_number;
	delete button_warning;
	delete label_ss_error;
	delete button_select;
	delete button_open;
	delete window_savestate_screen;
	delete icon_savestate;
	delete image_savestate;
	delete radioButton_onscreen_hide;
	delete radioButton_onscreen_show;
	delete window_onscreen_control;
	delete radioButton_custom_control_on;
	delete radioButton_custom_control_off;
	delete window_custom_control_enable;
	delete radioButton_control_config_1;
	delete radioButton_control_config_2;
	delete radioButton_control_config_3;
	delete radioButton_control_config_4;
	delete window_control_config;
	delete label1_control_config;
	delete label2_control_config;
	delete radioButton_dpad_custom;
	delete radioButton_dpad_joystick;
	delete radioButton_dpad_mouse;
	delete window_dpad;
	delete radioButton_joy_port0;
	delete radioButton_joy_port1;
	delete radioButton_joy_both;
	delete window_joystick;
	delete label1_sound;
	delete label2_sound;
	delete radioButton_statusln_on;
	delete radioButton_statusln_off;
	delete window_statusln;
	delete radioButton_autofirerate_light;
	delete radioButton_autofirerate_medium;
	delete radioButton_autofirerate_heavy;
	delete window_autofirerate;
	delete radioButton_mouseMultiplier_25;
	delete radioButton_mouseMultiplier_50;
	delete radioButton_mouseMultiplier_1;
	delete radioButton_mouseMultiplier_2;
	delete radioButton_mouseMultiplier_4;
	delete window_mouseMultiplier;
	delete radioButton_tapDelay_10;
	delete radioButton_tapDelay_5;
	delete radioButton_tapDelay_2;
	delete window_tapDelay;
    }
}

int run_mainMenuGuichan()
{
  	static int c=0;
	int end;
	int old_sound_rate = sound_rate;
	int old_stereo = mainMenu_soundStereo;
	mainMenu_case=-1;
	running=true;
	init_text(0);
#ifdef ANDROIDSDL	  
SDL_ANDROID_SetScreenKeyboardShown(0);
#endif
running = true;
        sdl::init();
        widgets::init();
        sdl::run();
        widgets::halt();
        sdl::halt();
    return mainMenu_case;
}

